#!/usr/bin/env python3
# ------------------------------------------------------------------
#
#   Copyright (C) 2013-2015 Canonical Ltd.
#   Authors: Steve Beattie <steve@nxnw.org>
#            Tyler Hicks <tyhicks@canonical.com>
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of version 2 of the GNU General Public
#   License published by the Free Software Foundation.
#
# ------------------------------------------------------------------

# TODO
# - check cache not used if parser in $PATH is newer
# - check cache used for force-complain, disable symlink, etc.

import os
import shutil
import sys
import tempfile
import unittest
from argparse import ArgumentParser

import testlib

ABSTRACTION_CONTENTS = '''
  # Simple example abstraction
  capability setuid,
'''
ABSTRACTION = 'suid-abstraction'

PROFILE_CONTENTS = '''
# Simple example profile for caching tests

/bin/pingy {
  #include <%s>
  capability net_raw,
  network inet raw,

  /bin/ping mixr,
  /etc/modules.conf r,
}
''' % (ABSTRACTION,)
PROFILE = 'sbin.pingy'
config = None


class AAParserCachingCommon(testlib.AATestTemplate):
    do_cleanup = True

    def setUp(self):
        """setup for each test"""
        global config

        # REPORT ALL THE OUTPUT
        self.maxDiff = None

        self.tmp_dir = tempfile.mkdtemp(prefix='aa-caching-')
        os.chmod(self.tmp_dir, 0o755)

        # write our sample abstraction and profile out
        self.abstraction = testlib.write_file(self.tmp_dir, ABSTRACTION, ABSTRACTION_CONTENTS)
        self.profile = testlib.write_file(self.tmp_dir, PROFILE, PROFILE_CONTENTS)

        if config.debug:
            self.do_cleanup = False
            self.debug = True

        # Warnings break the test harness, but chroots may not be setup
        # to have the config file, etc.
        self.cmd_prefix = [config.parser, '--config-file=./parser.conf', '--base', self.tmp_dir, '--skip-kernel-load']

        if not self.is_apparmorfs_mounted():
            self.cmd_prefix.extend(('-M', './features_files/features.all'))

        # Otherwise get_cache_dir() will try to create /var/cache/apparmor
        # and will fail when the test suite is run as non-root.
        self.cmd_prefix.extend((
            '--cache-loc', os.path.join(self.tmp_dir, 'cache')
        ))

        # create directory for cached blobs
        # NOTE: get_cache_dir() requires cmd_prefix to be fully initialized
        self.cache_dir = self.get_cache_dir(create=True)

        # default path of the output cache file
        self.cache_file = os.path.join(self.cache_dir, PROFILE)

    def tearDown(self):
        """teardown for each test"""

        if not self.do_cleanup:
            print("\n===> Skipping cleanup, leaving testfiles behind in '{}'".format(self.tmp_dir))
        else:
            if os.path.exists(self.tmp_dir):
                shutil.rmtree(self.tmp_dir)

    def get_cache_dir(self, create=False):
        cmd = [config.parser, '--print-cache-dir']
        cmd.extend(self.cmd_prefix)
        rc, report = self.run_cmd(cmd)
        if rc != 0:
            if "unrecognized option '--print-cache-dir'" not in report:
                self.fail('Unknown apparmor_parser error:\n' + report)

            cache_dir = os.path.join(self.tmp_dir, 'cache')
        else:
            cache_dir = report.strip()

        if create:
            os.makedirs(cache_dir)

        return cache_dir

    def assert_path_exists(self, path, expected=True):
        if expected:
            self.assertTrue(os.path.exists(path),
                            'test did not create file {}, when it was expected to do so'.format(path))
        else:
            self.assertFalse(os.path.exists(path),
                             'test created file {}, when it was not expected to do so'.format(path))

    def is_apparmorfs_mounted(self):
        return os.path.exists("/sys/kernel/security/apparmor")

    def require_apparmorfs(self):
        # skip the test if apparmor securityfs isn't mounted
        if not self.is_apparmorfs_mounted():
            raise unittest.SkipTest("WARNING: /sys/kernel/security/apparmor does not exist. Skipping test.")

    def compare_features_file(self, features_path, expected=True):
        # tests that need this function should call require_apparmorfs() early

        # compare features contents
        expected_output = testlib.read_features_dir('/sys/kernel/security/apparmor/features')
        with open(features_path) as f:
            features = f.read()
        if expected:
            self.assertEqual(
                expected_output, features,
                "features contents differ, expected:\n{}\nresult:\n{}".format(expected_output, features))
        else:
            self.assertNotEqual(
                expected_output, features,
                "features contents equal, expected:\n{}\nresult:\n{}".format(expected_output, features))


class AAParserBasicCachingTests(AAParserCachingCommon):

    def test_no_cache_by_default(self):
        """test profiles are not cached by default"""

        cmd = list(self.cmd_prefix)
        cmd.extend(('-q', '-r', self.profile))
        self.run_cmd_check(cmd)
        self.assert_path_exists(os.path.join(self.cache_dir, PROFILE), expected=False)

    def test_no_cache_w_skip_cache(self):
        """test profiles are not cached with --skip-cache"""

        cmd = list(self.cmd_prefix)
        cmd.extend(('-q', '--write-cache', '--skip-cache', '-r', self.profile))
        self.run_cmd_check(cmd)
        self.assert_path_exists(os.path.join(self.cache_dir, PROFILE), expected=False)

    def test_cache_when_requested(self):
        """test profiles are cached when requested"""

        cmd = list(self.cmd_prefix)
        cmd.extend(('-q', '--write-cache', '-r', self.profile))
        self.run_cmd_check(cmd)
        self.assert_path_exists(os.path.join(self.cache_dir, PROFILE))

    def test_write_features_when_caching(self):
        """test features file is written when caching"""

        cmd = list(self.cmd_prefix)
        cmd.extend(('-q', '--write-cache', '-r', self.profile))
        self.run_cmd_check(cmd)
        self.assert_path_exists(os.path.join(self.cache_dir, PROFILE))
        self.assert_path_exists(os.path.join(self.cache_dir, '.features'))

    def test_features_match_when_caching(self):
        """test features file is written when caching"""

        self.require_apparmorfs()

        cmd = list(self.cmd_prefix)
        cmd.extend(('-q', '--write-cache', '-r', self.profile))
        self.run_cmd_check(cmd)
        self.assert_path_exists(os.path.join(self.cache_dir, PROFILE))
        self.assert_path_exists(os.path.join(self.cache_dir, '.features'))

        self.compare_features_file(os.path.join(self.cache_dir, '.features'))


class AAParserAltCacheBasicTests(AAParserBasicCachingTests):
    """Same tests as above, but with an alternate cache location specified on the command line"""

    def setUp(self):
        super().setUp()

        alt_cache_loc = tempfile.mkdtemp(prefix='aa-alt-cache', dir=self.tmp_dir)
        os.chmod(alt_cache_loc, 0o755)

        self.unused_cache_loc = self.cache_dir
        self.cmd_prefix.extend(('--cache-loc', alt_cache_loc))
        self.cache_dir = self.get_cache_dir()

    def tearDown(self):
        if os.listdir(self.unused_cache_loc):
            self.fail("original cache dir '{}' not empty".format(self.unused_cache_loc))
        super().tearDown()


class AAParserCreateCacheBasicTestsCacheExists(AAParserBasicCachingTests):
    """Same tests as above, but with create cache option on the command line and the cache already exists"""

    def setUp(self):
        super().setUp()
        self.cmd_prefix.append('--create-cache-dir')


class AAParserCreateCacheBasicTestsCacheNotExist(AAParserBasicCachingTests):
    """Same tests as above, but with create cache option on the command line and cache dir removed"""

    def setUp(self):
        super().setUp()
        shutil.rmtree(self.cache_dir)
        self.cmd_prefix.append('--create-cache-dir')


class AAParserCreateCacheAltCacheTestsCacheNotExist(AAParserBasicCachingTests):
    """Same tests as above, but with create cache option on the command line,
       alt cache specified, and cache dir removed"""

    def setUp(self):
        super().setUp()
        shutil.rmtree(self.cache_dir)
        self.cmd_prefix.append('--create-cache-dir')


class AAParserCachingTests(AAParserCachingCommon):

    def setUp(self):
        super().setUp()

        r = testlib.filesystem_time_resolution()
        self.mtime_res = r[1]

    def _generate_cache_file(self):

        cmd = list(self.cmd_prefix)
        cmd.extend(('-q', '--write-cache', '-r', self.profile))
        self.run_cmd_check(cmd)
        self.assert_path_exists(self.cache_file)

    def _set_mtime(self, path, mtime):
        atime = os.stat(path).st_atime
        os.utime(path, (atime, mtime))
        self.assertEqual(os.stat(path).st_mtime, mtime)

    def test_cache_loaded_when_exists(self):
        """test cache is loaded when it exists, is newer than profile, and features match"""

        self._generate_cache_file()

        cmd = list(self.cmd_prefix)
        cmd.extend(('-v', '-r', self.profile))
        self.run_cmd_check(cmd, expected_string='Cached reload succeeded')

    def test_cache_not_loaded_when_skip_arg(self):
        """test cache is not loaded when --skip-cache is passed"""

        self._generate_cache_file()

        cmd = list(self.cmd_prefix)
        cmd.extend(('-v', '--skip-cache', '-r', self.profile))
        self.run_cmd_check(cmd, expected_string='Replacement succeeded for')

    def test_cache_not_loaded_when_skip_read_arg(self):
        """test cache is not loaded when --skip-read-cache is passed"""

        self._generate_cache_file()

        cmd = list(self.cmd_prefix)
        cmd.extend(('-v', '--skip-read-cache', '-r', self.profile))
        self.run_cmd_check(cmd, expected_string='Replacement succeeded for')

    def test_cache_not_loaded_when_features_differ(self):
        """test cache is not loaded when features file differs"""

        self._generate_cache_file()

        testlib.write_file(self.cache_dir, '.features', 'monkey\n')

        cmd = list(self.cmd_prefix)
        cmd.extend(('-v', '-r', self.profile))
        self.run_cmd_check(cmd, expected_string='Replacement succeeded for')

    def test_cache_writing_does_not_overwrite_features_when_features_differ(self):
        """test cache writing does not overwrite the features files when it differs and --skip-bad-cache is given"""

        self.require_apparmorfs()

        features_file = testlib.write_file(self.cache_dir, '.features', 'monkey\n')

        cmd = list(self.cmd_prefix)
        cmd.extend(('-v', '--write-cache', '--skip-bad-cache', '-r', self.profile))
        self.run_cmd_check(cmd, expected_string='Replacement succeeded for')
        self.assert_path_exists(features_file)
        # ensure that the features does *not* match the current features set
        self.compare_features_file(features_file, expected=False)

    def test_cache_writing_skipped_when_features_differ(self):
        """test cache writing is skipped when features file differs"""

        testlib.write_file(self.cache_dir, '.features', 'monkey\n')

        cmd = list(self.cmd_prefix)
        cmd.extend(('-v', '--write-cache', '--skip-bad-cache', '-r', self.profile))
        self.run_cmd_check(cmd, expected_string='Replacement succeeded for')
        self.assert_path_exists(self.cache_file, expected=False)

    def test_cache_writing_collision_of_features(self):
        """test cache writing collision of features"""
        # cache dir with different features causes a collision resulting
        # in a new cache dir
        self.require_apparmorfs()

        features_file = testlib.write_file(self.cache_dir, '.features', 'monkey\n')
        new_file = self.get_cache_dir()
        new_features_file = new_file + '/.features'

        cmd = list(self.cmd_prefix)
        cmd.extend(('-v', '--write-cache', '-r', self.profile))
        self.run_cmd_check(cmd, expected_string='Replacement succeeded for')
        self.assert_path_exists(features_file)
        self.assert_path_exists(new_features_file)
        self.compare_features_file(new_features_file)

    def test_cache_writing_updates_cache_file(self):
        """test cache writing updates cache file"""

        cache_file = testlib.write_file(self.cache_dir, PROFILE, 'monkey\n')
        orig_stat = os.stat(cache_file)

        cmd = list(self.cmd_prefix)
        cmd.extend(('-v', '--write-cache', '-r', self.profile))
        self.run_cmd_check(cmd, expected_string='Replacement succeeded for')
        self.assert_path_exists(cache_file)
        stat = os.stat(cache_file)
        # We check sizes here rather than whether the string monkey is
        # in cache_contents because of the difficulty coercing cache
        # file bytes into strings in python3
        self.assertNotEqual(orig_stat.st_size, stat.st_size, 'Expected cache file to be updated, size is not changed.')
        self.assertEqual(os.stat(self.profile).st_mtime, stat.st_mtime)

    def test_cache_writing_clears_all_files(self):
        """test cache writing clears all cache files"""

        check_file = testlib.write_file(self.cache_dir, 'monkey', 'monkey\n')

        cmd = list(self.cmd_prefix)
        cmd.extend(('-v', '--write-cache', '-r', self.profile))
        self.run_cmd_check(cmd, expected_string='Replacement succeeded for')
        self.assert_path_exists(check_file, expected=False)

    def test_profile_mtime_preserved(self):
        """test profile mtime is preserved when it is newest"""
        expected = 1
        self._set_mtime(self.abstraction, 0)
        self._set_mtime(self.profile, expected)
        self._generate_cache_file()
        self.assertEqual(expected, os.stat(self.cache_file).st_mtime)

    def test_abstraction_mtime_preserved(self):
        """test abstraction mtime is preserved when it is newest"""
        expected = 1000
        self._set_mtime(self.profile, 0)
        self._set_mtime(self.abstraction, expected)
        self._generate_cache_file()
        self.assertEqual(expected, os.stat(self.cache_file).st_mtime)

    def test_equal_mtimes_preserved(self):
        """test equal profile and abstraction mtimes are preserved"""
        expected = 10000 + self.mtime_res
        self._set_mtime(self.profile, expected)
        self._set_mtime(self.abstraction, expected)
        self._generate_cache_file()
        self.assertEqual(expected, os.stat(self.cache_file).st_mtime)

    def test_profile_newer_skips_cache(self):
        """test cache is skipped if profile is newer"""

        self._generate_cache_file()
        profile_mtime = os.stat(self.cache_file).st_mtime + self.mtime_res
        self._set_mtime(self.profile, profile_mtime)

        orig_stat = os.stat(self.cache_file)

        cmd = list(self.cmd_prefix)
        cmd.extend(('-v', '-r', self.profile))
        self.run_cmd_check(cmd, expected_string='Replacement succeeded for')

        stat = os.stat(self.cache_file)
        self.assertEqual(orig_stat.st_size, stat.st_size)
        self.assertEqual(orig_stat.st_ino, stat.st_ino)
        self.assertEqual(orig_stat.st_mtime, stat.st_mtime)

    def test_abstraction_newer_skips_cache(self):
        """test cache is skipped if abstraction is newer"""

        self._generate_cache_file()
        abstraction_mtime = os.stat(self.cache_file).st_mtime + self.mtime_res
        self._set_mtime(self.abstraction, abstraction_mtime)

        orig_stat = os.stat(self.cache_file)

        cmd = list(self.cmd_prefix)
        cmd.extend(('-v', '-r', self.profile))
        self.run_cmd_check(cmd, expected_string='Replacement succeeded for')

        stat = os.stat(self.cache_file)
        self.assertEqual(orig_stat.st_size, stat.st_size)
        self.assertEqual(orig_stat.st_ino, stat.st_ino)
        self.assertEqual(orig_stat.st_mtime, stat.st_mtime)

    def test_profile_newer_rewrites_cache(self):
        """test cache is rewritten if profile is newer"""

        self._generate_cache_file()
        profile_mtime = os.stat(self.cache_file).st_mtime + self.mtime_res
        self._set_mtime(self.profile, profile_mtime)

        orig_stat = os.stat(self.cache_file)

        cmd = list(self.cmd_prefix)
        cmd.extend(('-v', '-r', '-W', self.profile))
        self.run_cmd_check(cmd, expected_string='Replacement succeeded for')

        stat = os.stat(self.cache_file)
        self.assertNotEqual(orig_stat.st_ino, stat.st_ino)
        self.assertEqual(profile_mtime, stat.st_mtime)

    def test_abstraction_newer_rewrites_cache(self):
        """test cache is rewritten if abstraction is newer"""

        self._generate_cache_file()
        abstraction_mtime = os.stat(self.cache_file).st_mtime + self.mtime_res
        self._set_mtime(self.abstraction, abstraction_mtime)

        orig_stat = os.stat(self.cache_file)

        cmd = list(self.cmd_prefix)
        cmd.extend(('-v', '-r', '-W', self.profile))
        self.run_cmd_check(cmd, expected_string='Replacement succeeded for')

        stat = os.stat(self.cache_file)
        self.assertNotEqual(orig_stat.st_ino, stat.st_ino)
        self.assertEqual(abstraction_mtime, stat.st_mtime)

    def test_parser_newer_uses_cache(self):
        """test cache is not skipped if parser is newer"""

        self._generate_cache_file()

        # copy parser
        os.mkdir(os.path.join(self.tmp_dir, 'parser'))
        new_parser = os.path.join(self.tmp_dir, 'parser', 'apparmor_parser')
        shutil.copy(config.parser, new_parser)
        self._set_mtime(new_parser, os.stat(self.cache_file).st_mtime + self.mtime_res)

        cmd = list(self.cmd_prefix)
        cmd[0] = new_parser
        cmd.extend(('-v', '-r', self.profile))
        self.run_cmd_check(cmd, expected_string='Cached reload succeeded for')

    def _purge_cache_test(self, location):

        cache_file = testlib.write_file(self.cache_dir, location, 'monkey\n')

        cmd = list(self.cmd_prefix)
        cmd.extend(('-v', '--purge-cache', '-r', self.profile))
        self.run_cmd_check(cmd)
        # no message is output
        self.assert_path_exists(cache_file, expected=False)

    def test_cache_purge_removes_features_file(self):
        """test cache --purge-cache removes .features file"""
        self._purge_cache_test('.features')

    def test_cache_purge_removes_cache_file(self):
        """test cache --purge-cache removes profile cache file"""
        self._purge_cache_test(PROFILE)

    def test_cache_purge_removes_other_cache_files(self):
        """test cache --purge-cache removes other cache files"""
        self._purge_cache_test('monkey')


class AAParserAltCacheTests(AAParserCachingTests):
    """Same tests as above, but with an alternate cache location specified on the command line"""
    check_orig_cache = True

    def setUp(self):
        super().setUp()

        alt_cache_loc = tempfile.mkdtemp(prefix='aa-alt-cache', dir=self.tmp_dir)
        os.chmod(alt_cache_loc, 0o755)

        self.orig_cache_dir = self.cache_dir
        self.cmd_prefix.extend(('--cache-loc', alt_cache_loc))
        self.cache_dir = self.get_cache_dir(create=True)
        self.cache_file = os.path.join(self.cache_dir, PROFILE)

    def tearDown(self):
        if self.check_orig_cache and os.listdir(self.orig_cache_dir):
            self.fail("original cache dir '{}' not empty".format(self.orig_cache_dir))
        super().tearDown()

    def test_cache_purge_leaves_original_cache_alone(self):
        """test cache purging only touches alt cache"""

        # skip tearDown check to ensure non-alt cache is empty
        self.check_orig_cache = False
        filelist = (PROFILE, '.features', 'monkey')

        for f in filelist:
            testlib.write_file(self.orig_cache_dir, f, 'monkey\n')

        self._purge_cache_test(PROFILE)

        for f in filelist:
            if not os.path.exists(os.path.join(self.orig_cache_dir, f)):
                self.fail('cache purge removed {}, was not supposed to'.format(os.path.join(self.orig_cache_dir, f)))


def main():
    global config
    p = ArgumentParser()
    p.add_argument('-p', '--parser', default=testlib.DEFAULT_PARSER, action="store", dest='parser')
    p.add_argument('-v', '--verbose', action="store_true", dest="verbose")
    p.add_argument('-d', '--debug', action="store_true", dest="debug")
    config = p.parse_args()

    verbosity = 2 if config.verbose else 1

    test_suite = unittest.TestSuite()
    test_suite.addTest(unittest.TestLoader().loadTestsFromTestCase(AAParserBasicCachingTests))
    test_suite.addTest(unittest.TestLoader().loadTestsFromTestCase(AAParserAltCacheBasicTests))
    test_suite.addTest(unittest.TestLoader().loadTestsFromTestCase(AAParserCreateCacheBasicTestsCacheExists))
    test_suite.addTest(unittest.TestLoader().loadTestsFromTestCase(AAParserCreateCacheBasicTestsCacheNotExist))
    test_suite.addTest(unittest.TestLoader().loadTestsFromTestCase(AAParserCreateCacheAltCacheTestsCacheNotExist))
    test_suite.addTest(unittest.TestLoader().loadTestsFromTestCase(AAParserCachingTests))
    test_suite.addTest(unittest.TestLoader().loadTestsFromTestCase(AAParserAltCacheTests))
    try:
        result = unittest.TextTestRunner(verbosity=verbosity).run(test_suite)
    except Exception:
        rc = 1
    else:
        rc = 0 if result.wasSuccessful() else 1
    return rc


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python3
# ------------------------------------------------------------------
#
#   Copyright (C) 2014 Canonical Ltd.
#   Author: Steve Beattie <steve@nxnw.org>
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of version 2 of the GNU General Public
#   License published by the Free Software Foundation.
#
# ------------------------------------------------------------------

import os
import sys
from argparse import ArgumentParser

from testlib import read_features_dir

DEFAULT_FEATURES_DIR = '/sys/kernel/security/apparmor/features'


def main():
    p = ArgumentParser()

    p.add_argument('fdir', action="store", nargs='?', metavar="features_dir",
                   default=DEFAULT_FEATURES_DIR, help="path to features directory")
    config = p.parse_args()

    if not os.path.exists(config.fdir):
        print('Unable to find apparmor features directory "{}"'.format(config.fdir), file=sys.stderr)
        return 1

    features = read_features_dir(config.fdir)
    print(features)

    return 0


if __name__ == "__main__":
    sys.exit(main())



#! /usr/bin/python3
# ------------------------------------------------------------------
#
#    Copyright (C) 2019 Otto Kekäläinen <otto@kekalainen.net>
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License published by the Free Software Foundation.
#
# ------------------------------------------------------------------

import atexit
import io
import os
import sys
import unittest

import apparmor.aa as aa
import apparmor.ui as aaui
from apparmor.common import DebugLogger
from apparmor.fail import enable_aa_exception_handler
from apparmor.translations import init_translation
from common_test import AATest, setup_aa, setup_all_loops


class AACliBootstrapTest(AATest):
    """
    Generic test of the core AppArmor Python libraries that all command
    line tools rely on.
    """
    def AASetup(self):
        # Redirect sys.stdout to a buffer
        sys.stdout = io.StringIO()

        global _, debug_logger

        enable_aa_exception_handler()
        _ = init_translation()
        atexit.register(aa.on_exit)
        debug_logger = DebugLogger('Test AA')
        debug_logger.debug('Starting test')

    def AATeardown(self):
        debug_logger.debug('Ended test')

    def test_loadincludes(self):
        self.assertEqual(aa.loadincludes(), None)

    def test_i18n(self):
        self.assertEqual('Test string - do not translate', _('Test string - do not translate'))

    def test_aa_conf(self):
        confdir = os.getenv('__AA_CONFDIR')
        if confdir:
            self.assertEqual(aa.conf.CONF_DIR, confdir)
        else:
            self.assertEqual(aa.conf.CONF_DIR, '/etc/apparmor')

    def test_aa_ui_info(self):
        aaui.UI_Info('Test string')
        self.assertEqual(sys.stdout.getvalue(), 'Test string\n')

    def test_aa_ui_info_json(self):
        aaui.set_json_mode({'settings': {}})
        sys.stdout.getvalue()
        aaui.UI_Info('Test string')
        self.assertEqual(
            sys.stdout.getvalue(),
            '{"dialog": "apparmor-json-version","data": "2.12"}\n{"dialog": "info","data": "Test string"}\n')
        aaui.set_text_mode()


setup_aa(aa)  # Wrapper for aa.init_aa()
setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)



#!/usr/bin/python3

import subprocess
import sys

# TODO: transform this script to a package to use local imports so that if called with ./aa-notify, we use ./apparmor.*
from apparmor import aa
from apparmor.logparser import ReadLog

from apparmor.translations import init_translation
_ = init_translation()


def create_userns(template_path, name, bin_path, profile_path, decision):
    with open(template_path, 'r') as f:
        profile_template = f.read()

    rule = 'userns create' if decision == 'allow' else 'audit deny userns create'
    profile = profile_template.format(rule=rule, name=name, path=bin_path)

    with open(profile_path, 'w') as file:
        file.write(profile)

    try:
        subprocess.run(['apparmor_parser', '-r', profile_path], check=True)
    except subprocess.CalledProcessError:
        exit(_('Cannot reload updated profile'))


def add_to_profile(rule, profile_name):
    aa.init_aa()
    aa.read_profiles()

    rule_type, rule_class = ReadLog('', '', '').get_rule_type(rule)

    rule_obj = rule_class.create_instance(rule)

    if profile_name not in aa.aa or profile_name not in aa.aa[profile_name]:
        exit(_('Cannot find {} in profiles').format(profile_name))
    aa.aa[profile_name][profile_name][rule_type].add(rule_obj, cleanup=True)

    # Save changes
    aa.write_profile_ui_feedback(profile_name)
    aa.reload_base(profile_name)


def usage(is_help):
    print('This tool is a low level tool - do not use it directly')
    print('{} create_userns <template_path> <name> <bin_path> <profile_path> <decision>'.format(sys.argv[0]))
    print('{} add_rule <rule> <profile_name>'.format(sys.argv[0]))

    if is_help:
        exit(0)
    else:
        exit(1)


def main():
    if len(sys.argv) > 1:
        command = sys.argv[1]
    else:
        command = None  # Handle the case where no command is provided

    if command == 'create_userns':
        if not len(sys.argv) == 7:
            usage(False)
        create_userns(sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5], sys.argv[6])
    elif command == 'add_rule':
        if not len(sys.argv) == 4:
            usage(False)
        add_to_profile(sys.argv[2], sys.argv[3])
    elif command == 'help':
        usage(True)
    else:
        usage(False)


if __name__ == '__main__':
    main()



#! /usr/bin/python3
# ------------------------------------------------------------------
#
#    Copyright (C) 2014 Canonical Ltd.
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License published by the Free Software Foundation.
#
# ------------------------------------------------------------------

import unittest

import apparmor.aa as aa
from apparmor.common import AppArmorBug, AppArmorException
from apparmor.regex import (
    RE_PROFILE_CAP, RE_PROFILE_DBUS, RE_PROFILE_MOUNT, RE_PROFILE_PTRACE, RE_PROFILE_SIGNAL,
    RE_PROFILE_START, parse_profile_start_line, re_match_include, RE_PROFILE_UNIX,
    RE_PROFILE_PIVOT_ROOT,
    re_match_include_parse, strip_parenthesis, strip_quotes)
from common_test import AATest, setup_aa, setup_all_loops


class AARegexTest(AATest):
    def _run_test(self, params, expected):
        return _regex_test(self, params, expected)


class AANamedRegexTest(AATest):
    def _run_test(self, line, expected):
        """Run a line through self.regex.search() and verify the result

        Keyword arguments:
        line -- the line to search
        expected -- False if the search isn't expected to match or, if the search
                    is expected to match, a tuple of expected match groups.
        """
        matches = self.regex.search(line)
        if not expected:
            self.assertFalse(matches)
            return

        self.assertTrue(matches)

        for exp in expected:
            match = matches.group(exp)
            if match:
                match = match
            self.assertEqual(match, expected[exp], 'Group {} mismatch in rule {}'.format(exp, line))


class AARegexHasComma(AATest):
    """Tests for apparmor.aa.RE_RULE_HAS_COMMA"""

    def _check(self, line, expected=True):
        result = aa.RE_RULE_HAS_COMMA.search(line)
        if expected:
            self.assertTrue(result, 'Couldn\'t find a comma in "{}"'.format(line))
        else:
            self.assertEqual(None, result, 'Found an unexpected comma in "{}"'.format(line))


regex_has_comma_testcases = (
    ('dbus send%s', 'simple'),
    ('dbus (r, w, bind, eavesdrop)%s', 'embedded parens 01'),
    ('dbus (r, w,, bind, eavesdrop) %s', 'embedded parens 02'),
    ('dbus (r, w,, ) %s', 'embedded parens 03'),
    ('dbus () %s', 'empty parens'),
    ('member={Hello,AddMatch,RemoveMatch,GetNameOwner,NameHasOwner,StartServiceByName} %s ', 'embedded curly braces 01'),
    ('member={Hello,,,,,,AddMatch,,,NameHasOwner,StartServiceByName} %s ', 'embedded curly braces 02'),
    ('member={,Hello,,,,,,AddMatch,,,NameHasOwner,} %s ', 'embedded curly braces 03'),
    ('member={} %s ', 'empty curly braces'),
    ('dbus send%s# this is a comment', 'comment 01'),
    ('dbus send%s# this is a comment,', 'comment 02'),
    ('audit "/tmp/foo, bar" rw%s', 'quotes 01'),
    ('audit "/tmp/foo, bar" rw%s # comment', 'quotes 02'),
    ('audit "/tmp/foo, # bar" rw%s', 'comment embedded in quote 01'),
    ('audit "/tmp/foo, # bar" rw%s # comment', 'comment embedded in quote 02'),

    # lifted from parser/tst/simple_tests/vars/vars_alternation_3.sd
    ('/does/not/@{BAR},exist,notexist} r%s', 'partial alternation'),

    ('signal%s', 'bare signal'),
    ('signal receive%s', 'simple signal'),
    ('signal (send, receive)%s', 'embedded parens signal 01'),
    ('signal (send, receive) set=(hup, quit)%s', 'embedded parens signal 02'),

    ('ptrace%s', 'bare ptrace'),
    ('ptrace trace%s', 'simple ptrace'),
    ('ptrace (tracedby, readby)%s', 'embedded parens ptrace 01'),
    ('ptrace (trace) peer=/usr/bin/foo%s', 'embedded parens ptrace 02'),

    ('pivot_root%s', 'bare pivot_root'),
    ('pivot_root /old%s', 'pivot_root with old'),
    ('pivot_root /old new%s', 'pivot_root with new'),
    ('pivot_root /old /new -> child%s', 'pivot_root with child'),

    ('unix%s', 'bare unix'),
    ('unix create%s', 'simple unix'),
    ('peer=(addr=@abad1dea,label=a_profile) %s ', 'peer parens and comma'),
    ('type=stream%s', 'unix type'),
    ('unix (connect, receive, send)%s', 'unix perms'),

    # the following fail due to inadequacies in the regex
    # ('dbus (r, w, %s', 'incomplete dbus action'),
    # ('member="{Hello,AddMatch,RemoveMatch, %s', 'incomplete {} regex'),  # also invalid policy
    # ('member={Hello,AddMatch,RemoveMatch, %s', 'incomplete {} regex'),  # also invalid policy when trailing comma exists

    # the following passes the tests, but variable declarations are
    # odd in that they *don't* allow trailing commas; commas at the end
    # of the line need to be quoted.
    # ('@{BAR}={bar,baz,blort %s', 'tricksy variable declaration')
    # ('@{BAR}="{bar,baz,blort," %s', 'tricksy variable declaration')
    # The following fails the no comma test, but is invalid
    # ('@{BAR}={bar,baz,blort, %s', 'tricksy variable declaration')
    # The following fails the comma test, because it's really a no comma situation
    # ('@{BAR}="{bar,baz,blort%s" ', 'tricksy variable declaration')
)


def setup_has_comma_testcases():
    i = 0
    for (test_string, description) in regex_has_comma_testcases:
        i += 1

        def stub_test_comma(self, test_string=test_string):
            self._check(test_string % ',')

        def stub_test_no_comma(self, test_string=test_string):
            self._check(test_string % ' ', False)

        stub_test_comma.__doc__ = "test {} (w/comma)".format(description)
        stub_test_no_comma.__doc__ = "test {} (no comma)".format(description)
        setattr(AARegexHasComma, 'test_comma_{}'.format(i), stub_test_comma)
        setattr(AARegexHasComma, 'test_no_comma_{}'.format(i), stub_test_no_comma)


class AARegexSplitComment(AATest):
    """Tests for RE_HAS_COMMENT_SPLIT"""

    def _check(self, line, expected, comment=None, not_comment=None):
        result = aa.RE_HAS_COMMENT_SPLIT.search(line)
        if expected:
            self.assertTrue(result, 'Couldn\'t find a comment in "{}"'.format(line))
            self.assertEqual(
                result.group('comment'), comment,
                'Expected comment "{}", got "{}"'.format(comment, result.group('comment')))
            self.assertEqual(
                result.group('not_comment'), not_comment,
                'Expected not comment "{}", got "{}"'.format(not_comment, result.group('not_comment')))
        else:
            self.assertEqual(None, result, 'Found an unexpected comment "{}" in "{}"'.format(
                "" if result is None else result.group('comment'), line))


# Tuples of (string, expected result), where expected result is False if
# the string should not be considered as having a comment, or a second
# tuple of the not comment and comment sections split apart
regex_split_comment_testcases = (
    ('dbus send # this is a comment', ('dbus send ', '# this is a comment')),
    ('dbus send member=no_comment', False),
    ('dbus send member=no_comment, ', False),
    ('audit "/tmp/foo, # bar" rw', False),
    ('audit "/tmp/foo, # bar" rw # comment', ('audit "/tmp/foo, # bar" rw ', '# comment')),
    ('file,', False),
    ('file, # bare', ('file, ', '# bare')),
    ('file /tmp/foo rw, # read-write', ('file /tmp/foo rw, ', '# read-write')),
    ('signal, # comment', ('signal, ', '# comment')),
    ('signal receive set=(usr1 usr2) peer=foo,', False),
    ('ptrace, # comment', ('ptrace, ', '# comment')),
    ('ptrace (trace read) peer=/usr/bin/foo,', False),
    ('pivot_root, # comment', ('pivot_root, ', '# comment')),
    ('pivot_root /old /new -> child,', False),
)


def setup_split_comment_testcases():
    i = 0
    for (test_string, result) in regex_split_comment_testcases:
        i += 1

        def stub_test(self, test_string=test_string, result=result):
            if result is False:
                self._check(test_string, False)
            else:
                self._check(test_string, True, not_comment=result[0], comment=result[1])

        stub_test.__doc__ = "test '{}'".format(test_string)
        setattr(AARegexSplitComment, 'test_split_comment_{}'.format(i), stub_test)


def _regex_test(self, line, expected):
    """Run a line through self.regex.search() and verify the result

    Keyword arguments:
    line -- the line to search
    expected -- False if the search isn't expected to match or, if the search
                is expected to match, a tuple of expected match groups with all
                of the strings stripped
    """
    result = self.regex.search(line)
    if not expected:
        self.assertFalse(result)
        return

    self.assertTrue(result)

    groups = result.groups()
    self.assertEqual(len(groups), len(expected))
    for (i, group) in enumerate(groups):
        if group:
            group = group.strip()
        self.assertEqual(group, expected[i], 'Group {} mismatch in rule {}'.format(i, line))


class AARegexCapability(AARegexTest):
    """Tests for RE_PROFILE_CAP"""

    def AASetup(self):
        self.regex = RE_PROFILE_CAP

    tests = (
        ('   capability net_raw,', (None, None, None, None, 'net_raw', 'net_raw', None)),
        ('capability     net_raw   ,  ', (None, None, None, None, 'net_raw', 'net_raw', None)),
        ('   capability,', (None, None, None, None, None, None, None)),
        ('   capability   ,  ', (None, None, None, None, None, None, None)),
        ('   capabilitynet_raw,', False),
        ('   priority=1 capability net_raw,', ('priority=1', '1', None, None, 'net_raw', 'net_raw', None)),
        ('priority=1 capability     net_raw   ,  ', ('priority=1', '1', None, None, 'net_raw', 'net_raw', None)),
        ('   priority=1 capability,', ('priority=1', '1', None, None, None, None, None)),
        ('   priority=1 capability   ,  ', ('priority=1', '1', None, None, None, None, None)),
        ('   priority=1 capabilitynet_raw,', False),
    )


class AARegexDbus(AARegexTest):
    """Tests for RE_PROFILE_DBUS"""

    def AASetup(self):
        self.regex = RE_PROFILE_DBUS

    tests = (
        ('   dbus,',                                  (None, None, None,    None, 'dbus,',                        None,                     None)),
        ('   audit dbus,',                            (None, None, 'audit', None, 'dbus,',                        None,                     None)),
        ('   dbus send member=no_comment,',           (None, None, None,    None, 'dbus send member=no_comment,', 'send member=no_comment', None)),
        ('   dbus send member=no_comment, # comment', (None, None, None,    None, 'dbus send member=no_comment,', 'send member=no_comment', '# comment')),

        ('   priority=-11 dbus,',                                  ('priority=-11', '-11', None,    None, 'dbus,',                        None,                     None)),
        ('   priority=-11 audit dbus,',                            ('priority=-11', '-11', 'audit', None, 'dbus,',                        None,                     None)),
        ('   priority=-11 dbus send member=no_comment,',           ('priority=-11', '-11', None,    None, 'dbus send member=no_comment,', 'send member=no_comment', None)),
        ('   priority=-11 dbus send member=no_comment, # comment', ('priority=-11', '-11', None,    None, 'dbus send member=no_comment,', 'send member=no_comment', '# comment')),

        ('   dbusdriver,', False),
        ('   audit dbusdriver,', False),
        ('   priority=-11 audit dbusdriver,', False),
    )


class AARegexMount(AARegexTest):
    """Tests for RE_PROFILE_MOUNT"""

    def AASetup(self):
        self.regex = RE_PROFILE_MOUNT

    tests = (
        ('   mount,',           (None, None, None,    None,   'mount,',   'mount',   None, None)),
        ('   audit mount,',     (None, None, 'audit', None,   'mount,',   'mount',   None, None)),
        ('   umount,',          (None, None, None,    None,   'umount,',  'umount',  None, None)),
        ('   audit umount,',    (None, None, 'audit', None,   'umount,',  'umount',  None, None)),
        ('   unmount,',         (None, None, None,    None,   'unmount,', 'unmount', None, None)),
        ('   audit unmount,',   (None, None, 'audit', None,   'unmount,', 'unmount', None, None)),
        ('   remount,',         (None, None, None,    None,   'remount,', 'remount', None, None)),
        ('   deny remount,',    (None, None, None,    'deny', 'remount,', 'remount', None, None)),

        ('   priority = 0 mount,',           ('priority = 0', '0', None,    None,   'mount,',   'mount',   None, None)),
        ('   priority = 0 audit mount,',     ('priority = 0', '0', 'audit', None,   'mount,',   'mount',   None, None)),
        ('   priority = 0 umount,',          ('priority = 0', '0', None,    None,   'umount,',  'umount',  None, None)),
        ('   priority = 0 audit umount,',    ('priority = 0', '0', 'audit', None,   'umount,',  'umount',  None, None)),
        ('   priority = 0 unmount,',         ('priority = 0', '0', None,    None,   'unmount,', 'unmount', None, None)),
        ('   priority = 0 audit unmount,',   ('priority = 0', '0', 'audit', None,   'unmount,', 'unmount', None, None)),
        ('   priority = 0 remount,',         ('priority = 0', '0', None,    None,   'remount,', 'remount', None, None)),
        ('   priority = 0 deny remount,',    ('priority = 0', '0', None,    'deny', 'remount,', 'remount', None, None)),

        ('   mount, # comment', (None, None, None,    None,   'mount,',   'mount',   None, '# comment')),
        ('   priority = 0 mount, # comment', ('priority = 0', '0', None,    None,   'mount,',   'mount',   None, '# comment')),

        ('   mountain,', False),
        ('   audit mountain,', False),
        ('   priority = 0 audit mountain,', False),
    )


class AARegexSignal(AARegexTest):
    """Tests for RE_PROFILE_SIGNAL"""

    def AASetup(self):
        self.regex = RE_PROFILE_SIGNAL

    tests = (
        ('   signal,',                                            (None, None, None,    None, 'signal,',                                            None,                                         None)),
        ('   audit signal,',                                      (None, None, 'audit', None, 'signal,',                                            None,                                         None)),
        ('   signal receive,',                                    (None, None, None,    None, 'signal receive,',                                    'receive',                                    None)),
        ('   signal (send, receive),',                            (None, None, None,    None, 'signal (send, receive),',                            '(send, receive)',                            None)),
        ('   audit signal (receive),',                            (None, None, 'audit', None, 'signal (receive),',                                  '(receive)',                                  None)),
        ('   signal (send, receive) set=(usr1 usr2),',            (None, None, None,    None, 'signal (send, receive) set=(usr1 usr2),',            '(send, receive) set=(usr1 usr2)',            None)),
        ('   signal send set=(hup, quit) peer=/usr/sbin/daemon,', (None, None, None,    None, 'signal send set=(hup, quit) peer=/usr/sbin/daemon,', 'send set=(hup, quit) peer=/usr/sbin/daemon', None)),

        ('   priority = -1 signal,',                                            ('priority = -1', '-1', None,    None, 'signal,',                                            None,                                         None)),
        ('   priority = -1 audit signal,',                                      ('priority = -1', '-1', 'audit', None, 'signal,',                                            None,                                         None)),
        ('   priority = -1 signal receive,',                                    ('priority = -1', '-1', None,    None, 'signal receive,',                                    'receive',                                    None)),
        ('   priority = -1 signal (send, receive),',                            ('priority = -1', '-1', None,    None, 'signal (send, receive),',                            '(send, receive)',                            None)),
        ('   priority = -1 audit signal (receive),',                            ('priority = -1', '-1', 'audit', None, 'signal (receive),',                                  '(receive)',                                  None)),
        ('   priority = -1 signal (send, receive) set=(usr1 usr2),',            ('priority = -1', '-1', None,    None, 'signal (send, receive) set=(usr1 usr2),',            '(send, receive) set=(usr1 usr2)',            None)),
        ('   priority = -1 signal send set=(hup, quit) peer=/usr/sbin/daemon,', ('priority = -1', '-1', None,    None, 'signal send set=(hup, quit) peer=/usr/sbin/daemon,', 'send set=(hup, quit) peer=/usr/sbin/daemon', None)),

        ('   signalling,', False),
        ('   audit signalling,', False),
        ('   priority = -1 audit signalling,', False),
        ('   signalling receive,', False),
    )


class AARegexPtrace(AARegexTest):
    """Tests for RE_PROFILE_PTRACE"""

    def AASetup(self):
        self.regex = RE_PROFILE_PTRACE

    tests = (
        #                                            priority    audit    allow  rule                                  rule details                   comment
        ('   ptrace,',                              (None, None, None,    None, 'ptrace,',                             None,                          None)),
        ('   audit ptrace,',                        (None, None, 'audit', None, 'ptrace,',                             None,                          None)),
        ('   ptrace trace,',                        (None, None, None,    None, 'ptrace trace,',                       'trace',                       None)),
        ('   ptrace (tracedby, readby),',           (None, None, None,    None, 'ptrace (tracedby, readby),',          '(tracedby, readby)',          None)),
        ('   audit ptrace (read),',                 (None, None, 'audit', None, 'ptrace (read),',                      '(read)',                      None)),
        ('   ptrace trace peer=/usr/sbin/daemon,',  (None, None, None,    None, 'ptrace trace peer=/usr/sbin/daemon,', 'trace peer=/usr/sbin/daemon', None)),

        ('   priority=100 ptrace,',                              ('priority=100', '100', None,    None, 'ptrace,',                             None,                          None)),
        ('   priority=100 audit ptrace,',                        ('priority=100', '100', 'audit', None, 'ptrace,',                             None,                          None)),
        ('   priority=100 ptrace trace,',                        ('priority=100', '100', None,    None, 'ptrace trace,',                       'trace',                       None)),
        ('   priority=100 ptrace (tracedby, readby),',           ('priority=100', '100', None,    None, 'ptrace (tracedby, readby),',          '(tracedby, readby)',          None)),
        ('   priority=100 audit ptrace (read),',                 ('priority=100', '100', 'audit', None, 'ptrace (read),',                      '(read)',                      None)),
        ('   priority=100 ptrace trace peer=/usr/sbin/daemon,',  ('priority=100', '100', None,    None, 'ptrace trace peer=/usr/sbin/daemon,', 'trace peer=/usr/sbin/daemon', None)),

        ('   ptraceback,', False),
        ('   audit ptraceback,', False),
        ('   priority=100 audit ptraceback,', False),
        ('   ptraceback trace,', False),
    )


class AARegexPivotRoot(AARegexTest):
    """Tests for RE_PROFILE_PIVOT_ROOT"""

    def AASetup(self):
        self.regex = RE_PROFILE_PIVOT_ROOT

    tests = (
        ('   pivot_root,',                                      (None, None, None,    None, 'pivot_root,',                                None,                                 None)),
        ('   audit pivot_root,',                                (None, None, 'audit', None, 'pivot_root,',                                None,                                 None)),
        ('   pivot_root oldroot=/new/old,',                     (None, None, None,    None, 'pivot_root oldroot=/new/old,',               'oldroot=/new/old',                   None)),
        ('   pivot_root oldroot=/new/old /new,',                (None, None, None,    None, 'pivot_root oldroot=/new/old /new,',          'oldroot=/new/old /new',              None)),
        ('   pivot_root oldroot=/new/old /new -> child,',       (None, None, None,    None, 'pivot_root oldroot=/new/old /new -> child,', 'oldroot=/new/old /new -> child',     None)),
        ('   audit pivot_root oldroot=/new/old /new -> child,', (None, None, 'audit', None, 'pivot_root oldroot=/new/old /new -> child,', 'oldroot=/new/old /new -> child',     None)),

        ('   priority=-100 pivot_root,',                                      ('priority=-100', '-100', None,    None, 'pivot_root,',                                None,                                 None)),
        ('   priority=-100 audit pivot_root,',                                ('priority=-100', '-100', 'audit', None, 'pivot_root,',                                None,                                 None)),
        ('   priority=-100 pivot_root oldroot=/new/old,',                     ('priority=-100', '-100', None,    None, 'pivot_root oldroot=/new/old,',               'oldroot=/new/old',                   None)),
        ('   priority=-100 pivot_root oldroot=/new/old /new,',                ('priority=-100', '-100', None,    None, 'pivot_root oldroot=/new/old /new,',          'oldroot=/new/old /new',              None)),
        ('   priority=-100 pivot_root oldroot=/new/old /new -> child,',       ('priority=-100', '-100', None,    None, 'pivot_root oldroot=/new/old /new -> child,', 'oldroot=/new/old /new -> child',     None)),
        ('   priority=-100 audit pivot_root oldroot=/new/old /new -> child,', ('priority=-100', '-100', 'audit', None, 'pivot_root oldroot=/new/old /new -> child,', 'oldroot=/new/old /new -> child',     None)),

        ('pivot_root', False),  # comma missing

        ('pivot_rootbeer,', False),
        ('pivot_rootbeer    ,  ', False),
        ('pivot_rootbeer, # comment', False),
        ('pivot_rootbeer /new,  ', False),
        ('pivot_rootbeer /new, # comment', False),
        ('priority=-100 pivot_rootbeer /new, # comment', False),
    )


class AARegexUnix(AARegexTest):
    """Tests for RE_PROFILE_UNIX"""

    def AASetup(self):
        self.regex = RE_PROFILE_UNIX

    tests = (
        ('   unix,',                                                                               (None, None, None,    None,    'unix,',                                      None,                                   None)),
        ('   audit unix,',                                                                         (None, None, 'audit', None,    'unix,',                                      None,                                   None)),
        ('   unix accept,',                                                                        (None, None, None,    None,    'unix accept,',                               'accept',                               None)),
        ('   allow unix connect,',                                                                 (None, None, None,    'allow', 'unix connect,',                              'connect',                              None)),
        ('   audit allow unix bind,',                                                              (None, None, 'audit', 'allow', 'unix bind,',                                 'bind',                                 None)),
        ('   deny unix bind,',                                                                     (None, None, None,    'deny',  'unix bind,',                                 'bind',                                 None)),
        ('unix peer=(label=@{profile_name}),',                                                     (None, None, None,    None,    'unix peer=(label=@{profile_name}),',         'peer=(label=@{profile_name})',         None)),
        ('unix (receive) peer=(label=unconfined),',                                                (None, None, None,    None,    'unix (receive) peer=(label=unconfined),',    '(receive) peer=(label=unconfined)',    None)),
        (' unix (getattr, shutdown) peer=(addr=none),',                                            (None, None, None,    None,    'unix (getattr, shutdown) peer=(addr=none),', '(getattr, shutdown) peer=(addr=none)', None)),
        ('unix (connect, receive, send) type=stream peer=(label=unconfined,addr="@/tmp/dbus-*"),', (None, None, None,    None,    'unix (connect, receive, send) type=stream peer=(label=unconfined,addr="@/tmp/dbus-*"),',
                                                                                                                                                                    '(connect, receive, send) type=stream peer=(label=unconfined,addr="@/tmp/dbus-*")',  # noqa: E127
                                                                                                                                                                                                            None)),  # noqa: E127

        ('   priority=1 unix,',                                                                               ('priority=1', '1', None,    None,    'unix,',                                      None,                                   None)),
        ('   priority=1 audit unix,',                                                                         ('priority=1', '1', 'audit', None,    'unix,',                                      None,                                   None)),
        ('   priority=1 unix accept,',                                                                        ('priority=1', '1', None,    None,    'unix accept,',                               'accept',                               None)),
        ('   priority=1 allow unix connect,',                                                                 ('priority=1', '1', None,    'allow', 'unix connect,',                              'connect',                              None)),
        ('   priority=1 audit allow unix bind,',                                                              ('priority=1', '1', 'audit', 'allow', 'unix bind,',                                 'bind',                                 None)),
        ('   priority=1 deny unix bind,',                                                                     ('priority=1', '1', None,    'deny',  'unix bind,',                                 'bind',                                 None)),
        ('priority=1 unix peer=(label=@{profile_name}),',                                                     ('priority=1', '1', None,    None,    'unix peer=(label=@{profile_name}),',         'peer=(label=@{profile_name})',         None)),
        ('priority=1 unix (receive) peer=(label=unconfined),',                                                ('priority=1', '1', None,    None,    'unix (receive) peer=(label=unconfined),',    '(receive) peer=(label=unconfined)',    None)),
        (' priority=1 unix (getattr, shutdown) peer=(addr=none),',                                            ('priority=1', '1', None,    None,    'unix (getattr, shutdown) peer=(addr=none),', '(getattr, shutdown) peer=(addr=none)', None)),
        ('priority=1 unix (connect, receive, send) type=stream peer=(label=unconfined,addr="@/tmp/dbus-*"),', ('priority=1', '1', None,    None,    'unix (connect, receive, send) type=stream peer=(label=unconfined,addr="@/tmp/dbus-*"),',
                                                                                                                                                                    '(connect, receive, send) type=stream peer=(label=unconfined,addr="@/tmp/dbus-*")',  # noqa: E127
                                                                                                                                                                                                            None)),  # noqa: E127

        ('unixlike', False),
        ('deny unixlike,', False),
        ('priority=1 deny unixlike,', False),
    )


class AANamedRegexProfileStart_2(AANamedRegexTest):
    """Tests for RE_PROFILE_START"""

    def AASetup(self):
        self.regex = RE_PROFILE_START

    tests = (
        ('/bin/foo ', False),  # no '{'
        ('/bin/foo /bin/bar', False),  # missing 'profile' keyword
        ('profile {', False),  # no attachment
        ('   profile foo bar /foo {', False),  # missing quotes around "foo bar"
        ('bin/foo {', False),  # not starting with '/'
        ('"bin/foo" {', False),  # not starting with '/', quoted version

        ('   /foo {',                      {'plainprofile': '/foo',   'namedprofile': None,        'attachment': None,     'flags': None,         'comment': None}),
        ('   "/foo" {',                    {'plainprofile': '"/foo"', 'namedprofile': None,        'attachment': None,     'flags': None,         'comment': None}),
        ('   profile /foo {',              {'plainprofile': None,     'namedprofile': '/foo',      'attachment': None,     'flags': None,         'comment': None}),
        ('   profile "/foo" {',            {'plainprofile': None,     'namedprofile': '"/foo"',    'attachment': None,     'flags': None,         'comment': None}),
        ('   profile foo /foo {',          {'plainprofile': None,     'namedprofile': 'foo',       'attachment': '/foo',   'flags': None,         'comment': None}),
        ('   profile foo /foo (audit) {',  {'plainprofile': None,     'namedprofile': 'foo',       'attachment': '/foo',   'flags': 'audit',      'comment': None}),
        ('   profile "foo" "/foo" {',      {'plainprofile': None,     'namedprofile': '"foo"',     'attachment': '"/foo"', 'flags': None,         'comment': None}),
        ('   profile "foo bar" /foo {',    {'plainprofile': None,     'namedprofile': '"foo bar"', 'attachment': '/foo',   'flags': None,         'comment': None}),
        ('   /foo (complain) {',           {'plainprofile': '/foo',   'namedprofile': None,        'attachment': None,     'flags': 'complain',   'comment': None}),
        ('   /foo flags=(complain) {',     {'plainprofile': '/foo',   'namedprofile': None,        'attachment': None,     'flags': 'complain',   'comment': None}),
        ('   /foo (complain) { # x',       {'plainprofile': '/foo',   'namedprofile': None,        'attachment': None,     'flags': 'complain',   'comment': '# x'}),
        ('   /foo flags = ( complain ){#', {'plainprofile': '/foo',   'namedprofile': None,        'attachment': None,     'flags': ' complain ', 'comment': '#'}),
        ('  @{foo} {',                     {'plainprofile': '@{foo}', 'namedprofile': None,        'attachment': None,     'flags': None,         'comment': None}),
        ('  profile @{foo} {',             {'plainprofile': None,     'namedprofile': '@{foo}',    'attachment': None,     'flags': None,         'comment': None}),
        ('  profile @{foo} /bar {',        {'plainprofile': None,     'namedprofile': '@{foo}',    'attachment': '/bar',   'flags': None,         'comment': None}),
        ('  profile foo @{bar} {',         {'plainprofile': None,     'namedprofile': 'foo',       'attachment': '@{bar}', 'flags': None,         'comment': None}),
        ('  profile @{foo} @{bar} {',      {'plainprofile': None,     'namedprofile': '@{foo}',    'attachment': '@{bar}', 'flags': None,         'comment': None}),

        ('   /foo {',                      {'plainprofile': '/foo',   'namedprofile': None,  'leadingspace': '   '}),
        ('/foo {',                         {'plainprofile': '/foo',   'namedprofile': None,  'leadingspace': ''}),
        ('   profile foo {',               {'plainprofile': None,     'namedprofile': 'foo', 'leadingspace': '   '}),
        ('profile foo {',                  {'plainprofile': None,     'namedprofile': 'foo', 'leadingspace': ''}),
    )


class Test_parse_profile_start_line(AATest):
    tests = (
        ('   /foo {',                     {'profile': '/foo',    'profile_keyword': False, 'plainprofile': '/foo',   'namedprofile': None,      'attachment': None,     'flags': None,         'comment': None}),
        ('   "/foo" {',                   {'profile': '/foo',    'profile_keyword': False, 'plainprofile': '/foo',   'namedprofile': None,      'attachment': None,     'flags': None,         'comment': None}),
        ('   profile /foo {',             {'profile': '/foo',    'profile_keyword': True,  'plainprofile': None,     'namedprofile': '/foo',    'attachment': None,     'flags': None,         'comment': None}),
        ('   profile "/foo" {',           {'profile': '/foo',    'profile_keyword': True,  'plainprofile': None,     'namedprofile': '/foo',    'attachment': None,     'flags': None,         'comment': None}),
        ('   profile foo /foo {',         {'profile': 'foo',     'profile_keyword': True,  'plainprofile': None,     'namedprofile': 'foo',     'attachment': '/foo',   'flags': None,         'comment': None}),
        ('   profile foo /foo (audit) {', {'profile': 'foo',     'profile_keyword': True,  'plainprofile': None,     'namedprofile': 'foo',     'attachment': '/foo',   'flags': 'audit',      'comment': None}),
        ('   profile "foo" "/foo" {',     {'profile': 'foo',     'profile_keyword': True,  'plainprofile': None,     'namedprofile': 'foo',     'attachment': '/foo',   'flags': None,         'comment': None}),
        ('   profile "foo bar" /foo {',   {'profile': 'foo bar', 'profile_keyword': True,  'plainprofile': None,     'namedprofile': 'foo bar', 'attachment': '/foo',   'flags': None,         'comment': None}),
        ('   /foo (complain) {',          {'profile': '/foo',    'profile_keyword': False, 'plainprofile': '/foo',   'namedprofile': None,      'attachment': None,     'flags': 'complain',   'comment': None}),
        ('   /foo flags=(complain) {',    {'profile': '/foo',    'profile_keyword': False, 'plainprofile': '/foo',   'namedprofile': None,      'attachment': None,     'flags': 'complain',   'comment': None}),
        ('   /foo flags = ( complain ){', {'profile': '/foo',    'profile_keyword': False, 'plainprofile': '/foo',   'namedprofile': None,      'attachment': None,     'flags': ' complain ', 'comment': None}),
        ('   /foo (complain) { # x',      {'profile': '/foo',    'profile_keyword': False, 'plainprofile': '/foo',   'namedprofile': None,      'attachment': None,     'flags': 'complain',   'comment': '# x'}),

        ('   /foo {',                     {'profile': '/foo',  'leadingspace': '   ',      'plainprofile': '/foo',   'namedprofile': None}),
        ('/foo {',                        {'profile': '/foo',  'leadingspace': None,       'plainprofile': '/foo',   'namedprofile': None}),
        ('   profile foo {',              {'profile': 'foo',   'leadingspace': '   ',      'plainprofile': None,     'namedprofile': 'foo'}),
        ('profile foo {',                 {'profile': 'foo',   'leadingspace': None,       'plainprofile': None,     'namedprofile': 'foo'}),
        ('  @{foo} {',                    {'profile': '@{foo}',                            'plainprofile': '@{foo}', 'namedprofile': None,      'attachment': None,     'flags': None,         'comment': None}),
        ('  profile @{foo} {',            {'profile': '@{foo}',                            'plainprofile': None,     'namedprofile': '@{foo}',  'attachment': None,     'flags': None,         'comment': None}),
        ('  profile @{foo} /bar {',       {'profile': '@{foo}',                            'plainprofile': None,     'namedprofile': '@{foo}',  'attachment': '/bar',   'flags': None,         'comment': None}),
        ('  profile foo @{bar} {',        {'profile': 'foo',                               'plainprofile': None,     'namedprofile': 'foo',     'attachment': '@{bar}', 'flags': None,         'comment': None}),
        ('  profile @{foo} @{bar} {',     {'profile': '@{foo}',                            'plainprofile': None,     'namedprofile': '@{foo}',  'attachment': '@{bar}', 'flags': None,         'comment': None}),
    )

    def _run_test(self, line, expected):
        matches = parse_profile_start_line(line, 'somefile')

        self.assertTrue(matches)

        for exp in expected:
            self.assertEqual(
                matches[exp], expected[exp],
                'Group {} mismatch in rule {}'.format(exp, line))


class TestInvalid_parse_profile_start_line(AATest):
    tests = (
        ('/bin/foo ', False),  # no '{'
        ('/bin/foo /bin/bar', False),  # missing 'profile' keyword
        ('profile {', False),  # no attachment
        ('   profile foo bar /foo {', False),  # missing quotes around "foo bar"
    )

    def _run_test(self, line, expected):
        with self.assertRaises(AppArmorBug):
            parse_profile_start_line(line, 'somefile')


class Test_re_match_include(AATest):
    tests = (
        # #include
        ('#include <abstractions/base>',            'abstractions/base'),  # magic path
        ('#include <abstractions/base> # comment',  'abstractions/base'),
        ('#include<abstractions/base>#comment',     'abstractions/base'),
        ('   #include    <abstractions/base>  ',    'abstractions/base'),
        ('#include "/foo/bar"',                     '/foo/bar'),  # absolute path
        ('#include "/foo/bar" # comment',           '/foo/bar'),
        ('#include "/foo/bar"#comment',             '/foo/bar'),
        ('   #include "/foo/bar"  ',                '/foo/bar'),
        # include (without #)
        ('include <abstractions/base>',            'abstractions/base'),  # magic path
        ('include <abstractions/base> # comment',  'abstractions/base'),
        ('include<abstractions/base>#comment',     'abstractions/base'),
        ('   include    <abstractions/base>  ',    'abstractions/base'),
        ('include "/foo/bar"',                     '/foo/bar'),  # absolute path
        ('include "/foo/bar" # comment',           '/foo/bar'),
        ('include "/foo/bar"#comment',             '/foo/bar'),
        ('   include "/foo/bar"  ',                '/foo/bar'),

        (' some #include <abstractions/base>',      None),  # non-matching
        ('  /etc/fstab r,',                         None),
        ('/usr/include r,',                         None),
        ('/include r,',                             None),
        (' #include if exists <abstractions/base>', None),  # include if exists
        (' #include if exists "/foo/bar"',          None),
    )

    def _run_test(self, params, expected):
        self.assertEqual(re_match_include(params), expected)


class TestInvalid_re_match_include(AATest):
    tests = (
        ('#include <>',                  AppArmorException),  # '#include'
        ('#include <  >',                AppArmorException),
        ('#include ""',                  AppArmorException),
        ('#include "  "',                AppArmorException),
        ('#include',                     AppArmorException),
        ('#include  ',                   AppArmorException),
        ('#include "foo"',               AppArmorException),  # LP: 1738880 (relative)
        ('#include "foo" # comment',     AppArmorException),
        ('#include "foo"#comment',       AppArmorException),
        ('   #include "foo"  ',          AppArmorException),
        ('#include "foo/bar"',           AppArmorException),
        ('#include "foo/bar" # comment', AppArmorException),
        ('#include "foo/bar"#comment',   AppArmorException),
        ('   #include "foo/bar"  ',      AppArmorException),
        ('#include foo',                 AppArmorException),  # LP: 1738879 (no quotes)
        ('#include foo/bar',             AppArmorException),
        ('#include /foo/bar',            AppArmorException),
        ('#include foo bar',             AppArmorException),  # LP: 1738877 (space in name)
        ('#include foo bar/baz',         AppArmorException),
        ('#include "foo bar"',           AppArmorException),
        ('#include /foo bar',            AppArmorException),
        ('#include "/foo bar"',          AppArmorException),
        ('#include "foo bar/baz"',       AppArmorException),

        ('include <>',                   AppArmorException),  # 'include'
        ('include <  >',                 AppArmorException),
        ('include ""',                   AppArmorException),
        ('include "  "',                 AppArmorException),
        ('include',                      AppArmorException),
        ('include  ',                    AppArmorException),
        ('include "foo"',                AppArmorException),  # LP: 1738880 (relative)
        ('include "foo" # comment',      AppArmorException),
        ('include "foo"#comment',        AppArmorException),
        ('   include "foo"  ',           AppArmorException),
        ('include "foo/bar"',            AppArmorException),
        ('include "foo/bar" # comment',  AppArmorException),
        ('include "foo/bar"#comment',    AppArmorException),
        ('   include "foo/bar"  ',       AppArmorException),
        ('include foo',                  AppArmorException),  # LP: 1738879 (no quotes)
        ('include foo/bar',              AppArmorException),
        ('include /foo/bar',             AppArmorException),
        ('include foo bar',              AppArmorException),  # LP: 1738877 (space in name)
        ('include foo bar/baz',          AppArmorException),
        ('include "foo bar"',            AppArmorException),
        ('include /foo bar',             AppArmorException),
        ('include "/foo bar"',           AppArmorException),
        ('include "foo bar/baz"',        AppArmorException),
    )

    def _run_test(self, params, expected):
        with self.assertRaises(expected):
            re_match_include(params)


class Test_re_match_include_parse(AATest):
    tests = (
        #                                                       path            if exists  magic path
        # #include
        ('#include <abstractions/base>',                      ('abstractions/base', False, True)),  # magic path
        ('#include <abstractions/base> # comment',            ('abstractions/base', False, True)),
        ('#include<abstractions/base>#comment',               ('abstractions/base', False, True)),
        ('   #include     <abstractions/base>  ',             ('abstractions/base', False, True)),
        ('#include "/foo/bar"',                               ('/foo/bar',          False, False)),  # absolute path
        ('#include "/foo/bar" # comment',                     ('/foo/bar',          False, False)),
        ('#include "/foo/bar"#comment',                       ('/foo/bar',          False, False)),
        ('   #include "/foo/bar"  ',                          ('/foo/bar',          False, False)),
        # include (without #)
        ('include <abstractions/base>',                       ('abstractions/base', False, True)),  # magic path
        ('include <abstractions/base> # comment',             ('abstractions/base', False, True)),
        ('include<abstractions/base>#comment',                ('abstractions/base', False, True)),
        ('   include     <abstractions/base>  ',              ('abstractions/base', False, True)),
        ('include "/foo/bar"',                                ('/foo/bar',          False, False)),  # absolute path
        ('include "/foo/bar" # comment',                      ('/foo/bar',          False, False)),
        ('include "/foo/bar"#comment',                        ('/foo/bar',          False, False)),
        ('   include "/foo/bar"  ',                           ('/foo/bar',          False, False)),
        # #include if exists
        ('#include if exists <abstractions/base>',            ('abstractions/base', True,  True)),  # magic path
        ('#include if exists <abstractions/base> # comment',  ('abstractions/base', True,  True)),
        ('#include if exists<abstractions/base>#comment',     ('abstractions/base', True,  True)),
        ('   #include    if     exists<abstractions/base>  ', ('abstractions/base', True,  True)),
        ('#include if exists "/foo/bar"',                     ('/foo/bar',          True,  False)),  # absolute path
        ('#include if exists "/foo/bar" # comment',           ('/foo/bar',          True,  False)),
        ('#include if exists "/foo/bar"#comment',             ('/foo/bar',          True,  False)),
        ('   #include if exists "/foo/bar"  ',                ('/foo/bar',          True,  False)),
        # include if exists (without #)
        ('include if exists <abstractions/base>',             ('abstractions/base', True,  True)),  # magic path
        ('include if exists <abstractions/base> # comment',   ('abstractions/base', True,  True)),
        ('include if exists<abstractions/base>#comment',      ('abstractions/base', True,  True)),
        ('   include    if     exists<abstractions/base>  ',  ('abstractions/base', True,  True)),
        ('include if exists "/foo/bar"',                      ('/foo/bar',          True,  False)),  # absolute path
        ('include if exists "/foo/bar" # comment',            ('/foo/bar',          True,  False)),
        ('include if exists "/foo/bar"#comment',              ('/foo/bar',          True,  False)),
        ('   include if exists "/foo/bar"  ',                 ('/foo/bar',          True,  False)),

        (' some #include if exists <abstractions/base>',      (None,                None,  None)),  # non-matching
        ('  /etc/fstab r,',                                   (None,                None,  None)),
        ('/usr/include r,',                                   (None,                None,  None)),
        ('/include r,',                                       (None,                None,  None)),
        ('abi <abi/4.19>,',                                   (None,                None,  None)),  # abi rule
    )

    def _run_test(self, params, expected):
        self.assertEqual(re_match_include_parse(params, 'include'), expected)


class Test_re_match_include_parse_abi(AATest):
    tests = (
        #                                                   path     if exists  magic path
        ('abi <abi/4.19>,',                                ('abi/4.19',  False, True)),  # magic path
        ('abi <abi/4.19>, # comment',                      ('abi/4.19',  False, True)),
        ('   abi    <abi/4.19>   ,    #    comment',       ('abi/4.19',  False, True)),
        ('abi "/abi/4.19" ,',                              ('/abi/4.19', False, False)),  # quoted path starting with /
        ('abi "/abi/4.19",     #  comment',                ('/abi/4.19', False, False)),
        ('  abi     "/abi/4.19"    ,    #      comment  ', ('/abi/4.19', False, False)),
        ('  abi     "abi/4.19"    ,    #      comment  ',  ('abi/4.19',  False, False)),  # quoted path, no leading /
        ('abi abi/4.19,',                                  ('abi/4.19',  False, False)),  # without quotes
        ('some abi <abi/4.19>,',                           (None,        None,  None)),  # non-matching
        ('  /etc/fstab r,',                                (None,        None,  None)),
        ('/usr/abi r,',                                    (None,        None,  None)),
        ('/abi r,',                                        (None,        None,  None)),
        ('#include <abstractions/base>',                   (None,        None,  None)),  # include rule path
    )

    def _run_test(self, params, expected):
        self.assertEqual(re_match_include_parse(params, 'abi'), expected)


class Test_re_match_include_parse_errors(AATest):
    tests = (
        (('include <>', 'include'), AppArmorException),  # various rules with empty filename
        (('include ""', 'include'), AppArmorException),
        (('include   ', 'include'), AppArmorException),
        (('abi <>,',    'abi'),     AppArmorException),
        (('abi "",',    'abi'),     AppArmorException),
        (('abi   ,',    'abi'),     AppArmorException),
        (('abi <foo>,', 'invalid'), AppArmorBug),  # invalid rule name
    )

    def _run_test(self, params, expected):
        with self.assertRaises(expected):
            rule, rule_name = params
            re_match_include_parse(rule, rule_name)


class TestStripParenthesis(AATest):
    tests = (
        ('foo',      'foo'),
        ('(foo)',    'foo'),
        ('(  foo )', 'foo'),
        ('(foo',     '(foo'),
        ('foo  )',   'foo  )'),
        ('foo ()',   'foo ()'),
        ('()',       ''),
        ('(  )',     ''),
        ('(())',     '()'),
        (' (foo)',   '(foo)'),  # parenthesis not first char, whitespace stripped nevertheless
        ('(foo) ',   '(foo)'),  # parenthesis not last char, whitespace stripped nevertheless
    )

    def _run_test(self, params, expected):
        self.assertEqual(strip_parenthesis(params), expected)


class TestStripQuotes(AATest):
    tests = (
        ('foo',             'foo'),
        ('"foo"',           'foo'),
        ('"foo',            '"foo'),
        ('foo"',            'foo"'),
        ('""',              ''),
        ('foo"bar',         'foo"bar'),
        ('"foo"bar"',       'foo"bar'),
        ('""""foo"bar""""', '"""foo"bar"""'),
        ('',                ''),
        ('/',               '/'),
        ('"',               '"'),
    )

    def _run_test(self, params, expected):
        self.assertEqual(strip_quotes(params), expected)


setup_aa(aa)
setup_all_loops(__name__)
if __name__ == '__main__':
    # these two are not converted to a tests[] loop yet
    setup_has_comma_testcases()
    setup_split_comment_testcases()

    unittest.main(verbosity=1)



#! /usr/bin/python3
# ------------------------------------------------------------------
#
#    Copyright (C) 2011-2012 Canonical Ltd.
#    Copyright (C) 2019 Otto Kekäläinen
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License published by the Free Software Foundation.
#
# ------------------------------------------------------------------

import os
import pwd
import signal
import subprocess
import sys
import time
import unittest
from tempfile import NamedTemporaryFile
from datetime import datetime

import apparmor.aa as aa
from common_test import AATest, setup_aa, setup_all_loops

# The location of the aa-notify utility can be overridden by setting
# the APPARMOR_NOTIFY environment variable; this is useful for running
# these tests in an installed environment
aanotify_bin = ["../aa-notify"]


# http://www.chiark.greenend.org.uk/ucgi/~cjwatson/blosxom/2009-07-02-python-sigpipe.html
# This is needed so that the subprocesses that produce endless output
# actually quit when the reader goes away.
def subprocess_setup():
    # Python installs a SIGPIPE handler by default. This is usually not what
    # non-Python subprocesses expect.
    signal.signal(signal.SIGPIPE, signal.SIG_DFL)


def cmd(command):
    """Try to execute given command (array) and return its stdout, or return
    a textual error if it failed."""

    try:
        sp = subprocess.Popen(
            command,
            stdin=None,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            close_fds=True,
            preexec_fn=subprocess_setup
        )
    except OSError as e:
        return 127, str(e)

    stdout, stderr = sp.communicate(input)

    # If there was some error output, show that instead of stdout to ensure
    # test fails and does not mask potentially major warnings and errors.
    if stderr:
        out = stderr
    else:
        out = stdout

    return sp.returncode, out.decode('utf-8')


class AANotifyBase(AATest):

    def create_logfile_contents(_time):
        """Create temporary log file with 30 entries of different age"""

        test_logfile_contents_999_days_old = \
'''Feb  4 13:40:38 XPS-13-9370 kernel: [128552.834382] audit: type=1400 audit({epoch}:113): apparmor="ALLOWED" operation="exec" profile="libreoffice-soffice" name="/bin/uname" pid=4097 comm="sh" requested_mask="x" denied_mask="x" fsuid=1001 ouid=0 target="libreoffice-soffice//null-/bin/uname"
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.834888] audit: type=1400 audit({epoch}:114): apparmor="ALLOWED" operation="file_inherit" profile="libreoffice-soffice//null-/bin/uname" name="/dev/null" pid=4097 comm="uname" requested_mask="w" denied_mask="w" fsuid=1001 ouid=0
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.834890] audit: type=1400 audit({epoch}:115): apparmor="ALLOWED" operation="file_mmap" profile="libreoffice-soffice//null-/bin/uname" name="/bin/uname" pid=4097 comm="uname" requested_mask="rm" denied_mask="rm" fsuid=1001 ouid=0
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.835136] audit: type=1400 audit({epoch}:116): apparmor="ALLOWED" operation="file_mmap" profile="libreoffice-soffice//null-/bin/uname" name="/lib/x86_64-linux-gnu/ld-2.27.so" pid=4097 comm="uname" requested_mask="rm" denied_mask="rm" fsuid=1001 ouid=0
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.835377] audit: type=1400 audit({epoch}:117): apparmor="ALLOWED" operation="open" profile="libreoffice-soffice//null-/bin/uname" name="/etc/ld.so.cache" pid=4097 comm="uname" requested_mask="r" denied_mask="r" fsuid=1001 ouid=0
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.835405] audit: type=1400 audit({epoch}:118): apparmor="ALLOWED" operation="open" profile="libreoffice-soffice//null-/bin/uname" name="/lib/x86_64-linux-gnu/libc-2.27.so" pid=4097 comm="uname" requested_mask="r" denied_mask="r" fsuid=1001 ouid=0
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.835421] audit: type=1400 audit({epoch}:119): apparmor="ALLOWED" operation="file_mmap" profile="libreoffice-soffice//null-/bin/uname" name="/lib/x86_64-linux-gnu/libc-2.27.so" pid=4097 comm="uname" requested_mask="rm" denied_mask="rm" fsuid=1001 ouid=0
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.835696] audit: type=1400 audit({epoch}:120): apparmor="ALLOWED" operation="open" profile="libreoffice-soffice//null-/bin/uname" name="/usr/lib/locale/locale-archive" pid=4097 comm="uname" requested_mask="r" denied_mask="r" fsuid=1001 ouid=0
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.875891] audit: type=1400 audit({epoch}:121): apparmor="ALLOWED" operation="exec" profile="libreoffice-soffice" name="/usr/bin/file" pid=4111 comm="soffice.bin" requested_mask="x" denied_mask="x" fsuid=1001 ouid=0 target="libreoffice-soffice//null-/usr/bin/file"
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.880347] audit: type=1400 audit({epoch}:122): apparmor="ALLOWED" operation="file_mmap" profile="libreoffice-soffice//null-/usr/bin/file" name="/usr/bin/file" pid=4111 comm="file" requested_mask="rm" denied_mask="rm" fsuid=1001 ouid=0
'''.format(epoch=round(_time, 3) - 60 * 60 * 24 * 999)  # noqa: E128

        test_logfile_contents_30_days_old = \
'''Feb  4 13:40:38 XPS-13-9370 kernel: [128552.834382] audit: type=1400 audit({epoch}:113): apparmor="ALLOWED" operation="exec" profile="libreoffice-soffice" name="/bin/uname" pid=4097 comm="sh" requested_mask="x" denied_mask="x" fsuid=1001 ouid=0 target="libreoffice-soffice//null-/bin/uname"
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.834888] audit: type=1400 audit({epoch}:114): apparmor="ALLOWED" operation="file_inherit" profile="libreoffice-soffice//null-/bin/uname" name="/dev/null" pid=4097 comm="uname" requested_mask="w" denied_mask="w" fsuid=1001 ouid=0
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.834890] audit: type=1400 audit({epoch}:115): apparmor="ALLOWED" operation="file_mmap" profile="libreoffice-soffice//null-/bin/uname" name="/bin/uname" pid=4097 comm="uname" requested_mask="rm" denied_mask="rm" fsuid=1001 ouid=0
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.835136] audit: type=1400 audit({epoch}:116): apparmor="ALLOWED" operation="file_mmap" profile="libreoffice-soffice//null-/bin/uname" name="/lib/x86_64-linux-gnu/ld-2.27.so" pid=4097 comm="uname" requested_mask="rm" denied_mask="rm" fsuid=1001 ouid=0
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.835377] audit: type=1400 audit({epoch}:117): apparmor="ALLOWED" operation="open" profile="libreoffice-soffice//null-/bin/uname" name="/etc/ld.so.cache" pid=4097 comm="uname" requested_mask="r" denied_mask="r" fsuid=1001 ouid=0
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.835405] audit: type=1400 audit({epoch}:118): apparmor="ALLOWED" operation="open" profile="libreoffice-soffice//null-/bin/uname" name="/lib/x86_64-linux-gnu/libc-2.27.so" pid=4097 comm="uname" requested_mask="r" denied_mask="r" fsuid=1001 ouid=0
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.835421] audit: type=1400 audit({epoch}:119): apparmor="ALLOWED" operation="file_mmap" profile="libreoffice-soffice//null-/bin/uname" name="/lib/x86_64-linux-gnu/libc-2.27.so" pid=4097 comm="uname" requested_mask="rm" denied_mask="rm" fsuid=1001 ouid=0
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.835696] audit: type=1400 audit({epoch}:120): apparmor="ALLOWED" operation="open" profile="libreoffice-soffice//null-/bin/uname" name="/usr/lib/locale/locale-archive" pid=4097 comm="uname" requested_mask="r" denied_mask="r" fsuid=1001 ouid=0
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.875891] audit: type=1400 audit({epoch}:121): apparmor="ALLOWED" operation="exec" profile="libreoffice-soffice" name="/usr/bin/file" pid=4111 comm="soffice.bin" requested_mask="x" denied_mask="x" fsuid=1001 ouid=0 target="libreoffice-soffice//null-/usr/bin/file"
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.880347] audit: type=1400 audit({epoch}:122): apparmor="ALLOWED" operation="file_mmap" profile="libreoffice-soffice//null-/usr/bin/file" name="/usr/bin/file" pid=4111 comm="file" requested_mask="rm" denied_mask="rm" fsuid=1001 ouid=0
'''.format(epoch=round(_time, 3) - 60 * 60 * 24 * 30)  # noqa: E128

        test_logfile_contents_unrelevant_entries = \
'''Feb  1 19:35:44 XPS-13-9370 kernel: [99848.048761] audit: type=1400 audit(1549042544.968:72): apparmor="STATUS" operation="profile_load" profile="unconfined" name="/snap/core/6350/usr/lib/snapd/snap-confine" pid=12871 comm="apparmor_parser"
Feb  2 00:40:09 XPS-13-9370 kernel: [103014.549071] audit: type=1400 audit(1549060809.600:89): apparmor="STATUS" operation="profile_load" profile="unconfined" name="docker-default" pid=17195 comm="apparmor_parser"
Feb  4 20:05:42 XPS-13-9370 kernel: [132557.202931] audit: type=1400 audit(1549303542.661:136): apparmor="STATUS" operation="profile_replace" info="same as current profile, skipping" profile="unconfined" name="snap.atom.apm" pid=11306 comm="apparmor_parser"
'''  # noqa: E128

        test_logfile_contents_0_seconds_old = \
'''Feb  4 13:40:38 XPS-13-9370 kernel: [128552.834382] audit: type=1400 audit({epoch}:113): apparmor="ALLOWED" operation="exec" profile="libreoffice-soffice" name="/bin/uname" pid=4097 comm="sh" requested_mask="x" denied_mask="x" fsuid=1001 ouid=0 target="libreoffice-soffice//null-/bin/uname"
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.834888] audit: type=1400 audit({epoch}:114): apparmor="ALLOWED" operation="file_inherit" profile="libreoffice-soffice//null-/bin/uname" name="/dev/null" pid=4097 comm="uname" requested_mask="w" denied_mask="w" fsuid=1001 ouid=0
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.834890] audit: type=1400 audit({epoch}:115): apparmor="ALLOWED" operation="file_mmap" profile="libreoffice-soffice//null-/bin/uname" name="/bin/uname" pid=4097 comm="uname" requested_mask="rm" denied_mask="rm" fsuid=1001 ouid=0
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.835136] audit: type=1400 audit({epoch}:116): apparmor="ALLOWED" operation="file_mmap" profile="libreoffice-soffice//null-/bin/uname" name="/lib/x86_64-linux-gnu/ld-2.27.so" pid=4097 comm="uname" requested_mask="rm" denied_mask="rm" fsuid=1001 ouid=0
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.835377] audit: type=1400 audit({epoch}:117): apparmor="ALLOWED" operation="open" profile="libreoffice-soffice//null-/bin/uname" name="/etc/ld.so.cache" pid=4097 comm="uname" requested_mask="r" denied_mask="r" fsuid=1001 ouid=0
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.835405] audit: type=1400 audit({epoch}:118): apparmor="ALLOWED" operation="open" profile="libreoffice-soffice//null-/bin/uname" name="/lib/x86_64-linux-gnu/libc-2.27.so" pid=4097 comm="uname" requested_mask="r" denied_mask="r" fsuid=1001 ouid=0
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.835421] audit: type=1400 audit({epoch}:119): apparmor="ALLOWED" operation="file_mmap" profile="libreoffice-soffice//null-/bin/uname" name="/lib/x86_64-linux-gnu/libc-2.27.so" pid=4097 comm="uname" requested_mask="rm" denied_mask="rm" fsuid=1001 ouid=0
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.835696] audit: type=1400 audit({epoch}:120): apparmor="ALLOWED" operation="open" profile="libreoffice-soffice//null-/bin/uname" name="/usr/lib/locale/locale-archive" pid=4097 comm="uname" requested_mask="r" denied_mask="r" fsuid=1001 ouid=0
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.875891] audit: type=1400 audit({epoch}:121): apparmor="ALLOWED" operation="exec" profile="libreoffice-soffice" name="/usr/bin/file" pid=4111 comm="soffice.bin" requested_mask="x" denied_mask="x" fsuid=1001 ouid=0 target="libreoffice-soffice//null-/usr/bin/file"
Feb  4 13:40:38 XPS-13-9370 kernel: [128552.880347] audit: type=1400 audit({epoch}:122): apparmor="ALLOWED" operation="file_mmap" profile="libreoffice-soffice//null-/usr/bin/file" name="/usr/bin/file" pid=4111 comm="file" requested_mask="rm" denied_mask="rm" fsuid=1001 ouid=0
'''.format(epoch=round(_time, 3))  # noqa: E128

        return test_logfile_contents_999_days_old \
            + test_logfile_contents_30_days_old \
            + test_logfile_contents_unrelevant_entries \
            + test_logfile_contents_0_seconds_old

    @classmethod
    def setUpClass(cls):
        file_current = NamedTemporaryFile("w+", prefix='test-aa-notify-', delete=False)
        file_last_login = NamedTemporaryFile("w+", prefix='test-aa-notify-', delete=False)
        cls.test_logfile_current = file_current.name
        cls.test_logfile_last_login = file_last_login.name

        current_time_contents = cls.create_logfile_contents(time.time())
        file_current.write(current_time_contents)

        if os.path.isfile('/var/log/wtmp'):
            if os.name == "posix":
                username = pwd.getpwuid(os.geteuid()).pw_name
            else:
                username = os.environ.get('USER')
                if not username and hasattr(os, 'getlogin'):
                    username = os.getlogin()
            if 'SUDO_USER' in os.environ:
                username = os.environ.get('SUDO_USER')

            return_code, output = cmd(['last', username, '--fullnames', '--time-format', 'iso'])
            output = output.split('\n')[0]  # the first line is enough
            # example of output (util-linux last command):
            # ubuntu  tty7         :0               2024-01-05T14:29:11-03:00   gone - no logout
            # example of output (wtmpdb last command, local login):
            # ubuntu  tty7                          2025-01-15T09:32:49-0800   - still logged in
            # example of output (wtmpdb last command, remote login)
            # ubuntu  tty7         192.168.122.1    2024-01-05T14:29:11-03:00   gone - no logout
            # example of output (wtmpdb last command, login via lxd-agent)
            # ubuntu                                 2025-03-18T00:34:25+0000  - still logged in
            if output.startswith(username):
                for col in range(3, 1, -1):
                    try:
                        last_login = output.split()[col]
                        last_login_epoch = datetime.fromisoformat(last_login).timestamp()
                        # add 60 seconds to the epoch so that the time in the logs are AFTER login time
                        last_login_contents = cls.create_logfile_contents(last_login_epoch + 60)
                        file_last_login.write(last_login_contents)
                        break
                    except (IndexError, ValueError):
                        continue

    @classmethod
    def tearDownClass(cls):
        """Remove temporary log file after tests ended"""

        if cls.test_logfile_current and os.path.exists(cls.test_logfile_current):
            os.remove(cls.test_logfile_current)
        if cls.test_logfile_last_login and os.path.exists(cls.test_logfile_last_login):
            os.remove(cls.test_logfile_last_login)


class AANotifyTest(AANotifyBase):

    # The Perl aa-notify script was written so, that it will checked for kern.log
    # before printing help when invoked without arguments (sic!).
    @unittest.skipUnless(os.path.isfile('/var/log/kern.log'), 'Requires kern.log on system')
    def test_no_arguments(self):
        """Test using no arguments at all"""

        expected_return_code = 0
        expected_output_has = 'usage: aa-notify'

        return_code, output = cmd(aanotify_bin)
        result = 'Got return code {}, expected {}\n'.format(return_code, expected_return_code)
        self.assertEqual(expected_return_code, return_code, result + output)
        result = 'Got output "{}", expected "{}"\n'.format(output, expected_output_has)
        self.assertIn(expected_output_has, output, result + output)

    def test_help_contents(self):
        """Test output of help text"""

        expected_return_code = 0
        expected_output_1 = \
'''usage: aa-notify [-h] [-p] [--display DISPLAY] [--xauthority XAUTHORITY]
                 [-f FILE] [-l] [-s NUM] [-v] [-u USER] [-w NUM]
                 [--prompt-filter PF] [--debug] [--filter.profile PROFILE]
                 [--filter.operation OPERATION] [--filter.name NAME]
                 [--filter.denied DENIED] [--filter.family FAMILY]
                 [--filter.socket SOCKET]

Display AppArmor notifications or messages for DENIED entries.
'''  # noqa: E128

        expected_output_2 = \
'''
  -h, --help            show this help message and exit
  -p, --poll            poll AppArmor logs and display notifications
  --display DISPLAY     set the DISPLAY environment variable (might be needed if
                        sudo resets $DISPLAY)
  --xauthority XAUTHORITY
                        set the XAUTHORITY environment variable (might be needed
                        if sudo resets XAUTHORITY)
  -f, --file FILE       search FILE for AppArmor messages
  -l, --since-last      display stats since last login
  -s, --since-days NUM  show stats for last NUM days (can be used alone or with
                        -p)
  -v, --verbose         show messages with stats
  -u, --user USER       user to drop privileges to when not using sudo
  -w, --wait NUM        wait NUM seconds before displaying notifications (with
                        -p)
  --prompt-filter PF    kind of operations which display a popup prompt
  --debug               debug mode

Filtering options:
  Filters are used to reduce the output of information to only those entries
  that will match the filter. Filters use Python's regular expression syntax.

  --filter.profile PROFILE
                        regular expression to match the profile
  --filter.operation OPERATION
                        regular expression to match the operation
  --filter.name NAME    regular expression to match the name
  --filter.denied DENIED
                        regular expression to match the denied mask
  --filter.family FAMILY
                        regular expression to match the network family
  --filter.socket SOCKET
                        regular expression to match the network socket type
'''  # noqa: E128

        if sys.version_info[:2] < (3, 13):
            # Python 3.13 tweaked argparse output [1]. When running on older
            # Python versions, we adapt the expected output to match.
            #
            # https://github.com/python/cpython/pull/103372
            patches = [(
                ', --file FILE     ',
                ' FILE, --file FILE',
            ), (
                ', --since-days NUM  show stats for last NUM days (can be used alone or with',
                ' NUM, --since-days NUM\n'
                + '                        show stats for last NUM days (can be used alone or with',
            ), (
                ', --user USER     ',
                ' USER, --user USER',
            ), (
                ', --wait NUM    ',
                ' NUM, --wait NUM',
            )]
            for patch in patches:
                expected_output_2 = expected_output_2.replace(patch[0], patch[1])

        return_code, output = cmd(aanotify_bin + ['--help'])
        result = 'Got return code {}, expected {}\n'.format(return_code, expected_return_code)
        self.assertEqual(expected_return_code, return_code, result + output)

        self.assertIn(expected_output_1, output)
        self.assertIn(expected_output_2, output)

    def test_entries_since_100_days(self):
        """Test showing log entries since 100 days"""

        expected_return_code = 0
        expected_output_has = 'AppArmor denials: 20 (since'

        return_code, output = cmd(aanotify_bin + ['-f', self.test_logfile_current, '-s', '100'])
        result = 'Got return code {}, expected {}\n'.format(return_code, expected_return_code)
        self.assertEqual(expected_return_code, return_code, result + output)
        result = 'Got output "{}", expected "{}"\n'.format(output, expected_output_has)
        self.assertIn(expected_output_has, output, result + output)

    @unittest.skipUnless(os.path.isfile('/var/log/wtmp'), 'Requires wtmp on system')
    def test_entries_since_login(self):
        """Test showing log entries since last login"""

        expected_return_code = 0
        expected_output_has = 'AppArmor denials: 10 (since'

        return_code, output = cmd(aanotify_bin + ['-f', self.test_logfile_last_login, '-l'])
        if "ERROR: Could not find last login" in output:
            self.skipTest('Could not find last login')
        result = 'Got return code {}, expected {}\n'.format(return_code, expected_return_code)
        self.assertEqual(expected_return_code, return_code, result + output)
        result = 'Got output "{}", expected "{}"\n'.format(output, expected_output_has)
        self.assertIn(expected_output_has, output, result + output)

    @unittest.skipUnless(os.path.isfile('/var/log/wtmp'), 'Requires wtmp on system')
    def test_entries_since_login_verbose(self):
        """Test showing log entries since last login in verbose mode"""

        expected_return_code = 0
        expected_output_has = \
'''Profile: libreoffice-soffice
Operation: exec
Name: /bin/uname
Denied: x
Logfile: {logfile}

Profile: libreoffice-soffice//null-/bin/uname
Operation: file_inherit
Name: /dev/null
Denied: w
Logfile: {logfile}

Profile: libreoffice-soffice//null-/bin/uname
Operation: file_mmap
Name: /bin/uname
Denied: rm
Logfile: {logfile}

Profile: libreoffice-soffice//null-/bin/uname
Operation: file_mmap
Name: /lib/x86_64-linux-gnu/ld-2.27.so
Denied: rm
Logfile: {logfile}

Profile: libreoffice-soffice//null-/bin/uname
Operation: open
Name: /etc/ld.so.cache
Denied: r
Logfile: {logfile}

Profile: libreoffice-soffice//null-/bin/uname
Operation: open
Name: /lib/x86_64-linux-gnu/libc-2.27.so
Denied: r
Logfile: {logfile}

Profile: libreoffice-soffice//null-/bin/uname
Operation: file_mmap
Name: /lib/x86_64-linux-gnu/libc-2.27.so
Denied: rm
Logfile: {logfile}

Profile: libreoffice-soffice//null-/bin/uname
Operation: open
Name: /usr/lib/locale/locale-archive
Denied: r
Logfile: {logfile}

Profile: libreoffice-soffice
Operation: exec
Name: /usr/bin/file
Denied: x
Logfile: {logfile}

Profile: libreoffice-soffice//null-/usr/bin/file
Operation: file_mmap
Name: /usr/bin/file
Denied: rm
Logfile: {logfile}

AppArmor denials: 10 (since'''.format(logfile=self.test_logfile_last_login)  # noqa: E128

        return_code, output = cmd(aanotify_bin + ['-f', self.test_logfile_last_login, '-l', '-v'])
        if "ERROR: Could not find last login" in output:
            self.skipTest('Could not find last login')
        result = 'Got return code {}, expected {}\n'.format(return_code, expected_return_code)
        self.assertEqual(expected_return_code, return_code, result + output)
        result = 'Got output "{}", expected "{}"\n'.format(output, expected_output_has)
        self.assertIn(expected_output_has, output, result + output)


class AANotifyProfileFilterTest(AANotifyBase):

    def test_profile_regex_since_100_days(self):
        profile_tests = (
            (['--filter.profile', 'libreoffice'], (0, 'AppArmor denials: 20 (since')),
            (['--filter.profile', 'libreoffice-soffice'], (0, 'AppArmor denials: 20 (since')),
            (['--filter.profile', 'libreoffice-soffice$'], (0, 'AppArmor denials: 4 (since')),
            (['--filter.profile', '^libreoffice-soffice$'], (0, 'AppArmor denials: 4 (since')),
            (['--filter.profile', 'libreoffice-soffice//null-/bin/uname'], (0, 'AppArmor denials: 14 (since')),
            (['--filter.profile', 'uname'], (0, 'AppArmor denials: 0 (since')),
            (['--filter.profile', '.*uname'], (0, 'AppArmor denials: 14 (since')),
            (['--filter.profile', 'libreoffice-soffice//null-/.*'], (0, 'AppArmor denials: 16 (since')),
            (['--filter.profile', 'libreoffice-soffice//null-/foo'], (0, 'AppArmor denials: 0 (since')),
            (['--filter.profile', 'libreoffice-soffice/foo'], (0, 'AppArmor denials: 0 (since')),
            (['--filter.profile', 'bar'], (0, 'AppArmor denials: 0 (since')),
        )
        days_params = ['-f', self.test_logfile_current, '-s', '100']

        for test in profile_tests:
            params = test[0]
            expected = test[1]

            with self.subTest(params=params, expected=expected):
                expected_return_code = expected[0]
                expected_output_has = expected[1]

                return_code, output = cmd(aanotify_bin + days_params + params)
                result = 'Got return code {}, expected {}\n'.format(return_code, expected_return_code)
                self.assertEqual(expected_return_code, return_code, result + output)
                result = 'Got output "{}", expected "{}"\n'.format(output, expected_output_has)
                self.assertIn(expected_output_has, output, result + output)

    @unittest.skipUnless(os.path.isfile('/var/log/wtmp'), 'Requires wtmp on system')
    def test_profile_regex_since_login(self):
        profile_tests = (
            (['--filter.profile', 'libreoffice'], (0, 'AppArmor denials: 10 (since')),
            (['--filter.profile', 'libreoffice-soffice'], (0, 'AppArmor denials: 10 (since')),
            (['--filter.profile', 'libreoffice-soffice$'], (0, 'AppArmor denials: 2 (since')),
            (['--filter.profile', '^libreoffice-soffice$'], (0, 'AppArmor denials: 2 (since')),
            (['--filter.profile', 'libreoffice-soffice//null-/bin/uname'], (0, 'AppArmor denials: 7 (since')),
            (['--filter.profile', 'uname'], (0, 'AppArmor denials: 0 (since')),
            (['--filter.profile', '.*uname'], (0, 'AppArmor denials: 7 (since')),
            (['--filter.profile', 'libreoffice-soffice//null-/.*'], (0, 'AppArmor denials: 8 (since')),
            (['--filter.profile', 'libreoffice-soffice//null-/foo'], (0, 'AppArmor denials: 0 (since')),
            (['--filter.profile', 'libreoffice-soffice/foo'], (0, 'AppArmor denials: 0 (since')),
            (['--filter.profile', 'bar'], (0, 'AppArmor denials: 0 (since')),
        )
        login_params = ['-f', self.test_logfile_last_login, '-l']

        for test in profile_tests:
            params = test[0]
            expected = test[1]

            with self.subTest(params=params, expected=expected):
                expected_return_code = expected[0]
                expected_output_has = expected[1]

                return_code, output = cmd(aanotify_bin + login_params + params)
                if 'ERROR: Could not find last login' in output:
                    self.skipTest('Could not find last login')
                result = 'Got return code {}, expected {}\n'.format(return_code, expected_return_code)
                self.assertEqual(expected_return_code, return_code, result + output)
                result = 'Got output "{}", expected "{}"\n'.format(output, expected_output_has)
                self.assertIn(expected_output_has, output, result + output)


class AANotifyOperationFilterTest(AANotifyBase):

    def test_operation_regex_since_100_days(self):
        operation_tests = (
            (['--filter.operation', 'exec'], (0, 'AppArmor denials: 4 (since')),
            (['--filter.operation', 'file_inherit'], (0, 'AppArmor denials: 2 (since')),
            (['--filter.operation', 'file_mmap'], (0, 'AppArmor denials: 8 (since')),
            (['--filter.operation', 'open'], (0, 'AppArmor denials: 6 (since')),
            (['--filter.operation', 'file.*'], (0, 'AppArmor denials: 10 (since')),
            (['--filter.operation', 'profile_load'], (0, 'AppArmor denials: 0 (since')),
            (['--filter.operation', 'profile_replace'], (0, 'AppArmor denials: 0 (since')),
            (['--filter.operation', 'bar'], (0, 'AppArmor denials: 0 (since')),
            (['--filter.operation', 'userns_create'], (0, 'AppArmor denials: 0 (since')),
        )
        days_params = ['-f', self.test_logfile_current, '-s', '100']

        for test in operation_tests:
            params = test[0]
            expected = test[1]

            with self.subTest(params=params, expected=expected):
                expected_return_code = expected[0]
                expected_output_has = expected[1]

                return_code, output = cmd(aanotify_bin + days_params + params)
                result = 'Got return code {}, expected {}\n'.format(return_code, expected_return_code)
                self.assertEqual(expected_return_code, return_code, result + output)
                result = 'Got output "{}", expected "{}"\n'.format(output, expected_output_has)
                self.assertIn(expected_output_has, output, result + output)

    @unittest.skipUnless(os.path.isfile('/var/log/wtmp'), 'Requires wtmp on system')
    def test_operation_regex_since_login(self):
        operation_tests = (
            (['--filter.operation', 'exec'], (0, 'AppArmor denials: 2 (since')),
            (['--filter.operation', 'file_inherit'], (0, 'AppArmor denials: 1 (since')),
            (['--filter.operation', 'file_mmap'], (0, 'AppArmor denials: 4 (since')),
            (['--filter.operation', 'open'], (0, 'AppArmor denials: 3 (since')),
            (['--filter.operation', 'file.*'], (0, 'AppArmor denials: 5 (since')),
            (['--filter.operation', 'profile_load'], (0, 'AppArmor denials: 0 (since')),
            (['--filter.operation', 'profile_replace'], (0, 'AppArmor denials: 0 (since')),
            (['--filter.operation', 'bar'], (0, 'AppArmor denials: 0 (since')),
            (['--filter.operation', 'userns_create'], (0, 'AppArmor denials: 0 (since')),
        )
        login_params = ['-f', self.test_logfile_last_login, '-l']

        for test in operation_tests:
            params = test[0]
            expected = test[1]

            with self.subTest(params=params, expected=expected):
                expected_return_code = expected[0]
                expected_output_has = expected[1]

                return_code, output = cmd(aanotify_bin + login_params + params)
                if 'ERROR: Could not find last login' in output:
                    self.skipTest('Could not find last login')
                result = 'Got return code {}, expected {}\n'.format(return_code, expected_return_code)
                self.assertEqual(expected_return_code, return_code, result + output)
                result = 'Got output "{}", expected "{}"\n'.format(output, expected_output_has)
                self.assertIn(expected_output_has, output, result + output)


class AANotifyNameFilterTest(AANotifyBase):

    def test_name_regex_since_100_days(self):
        name_tests = (
            (['--filter.name', '/bin/uname'], (0, 'AppArmor denials: 4 (since')),
            (['--filter.name', '/dev/null'], (0, 'AppArmor denials: 2 (since')),
            (['--filter.name', '/lib/x86_64-linux-gnu/ld-2.27.so'], (0, 'AppArmor denials: 2 (since')),
            (['--filter.name', '/lib/x86_64-linux-gnu/libc-2.27.so'], (0, 'AppArmor denials: 4 (since')),
            (['--filter.name', '/etc/ld.so.cache'], (0, 'AppArmor denials: 2 (since')),
            (['--filter.name', '/usr/lib/locale/locale-archive'], (0, 'AppArmor denials: 2 (since')),
            (['--filter.name', '/usr/bin/file'], (0, 'AppArmor denials: 4 (since')),
            (['--filter.name', '/'], (0, 'AppArmor denials: 20 (since')),
            (['--filter.name', '/.*'], (0, 'AppArmor denials: 20 (since')),
            (['--filter.name', '.*bin.*'], (0, 'AppArmor denials: 8 (since')),
            (['--filter.name', '/(usr/)?bin.*'], (0, 'AppArmor denials: 8 (since')),
            (['--filter.name', '/foo'], (0, 'AppArmor denials: 0 (since')),
        )
        days_params = ['-f', self.test_logfile_current, '-s', '100']

        for test in name_tests:
            params = test[0]
            expected = test[1]

            with self.subTest(params=params, expected=expected):
                expected_return_code = expected[0]
                expected_output_has = expected[1]

                return_code, output = cmd(aanotify_bin + days_params + params)
                result = 'Got return code {}, expected {}\n'.format(return_code, expected_return_code)
                self.assertEqual(expected_return_code, return_code, result + output)
                result = 'Got output "{}", expected "{}"\n'.format(output, expected_output_has)
                self.assertIn(expected_output_has, output, result + output)

    @unittest.skipUnless(os.path.isfile('/var/log/wtmp'), 'Requires wtmp on system')
    def test_name_regex_since_login(self):
        name_tests = (
            (['--filter.name', '/bin/uname'], (0, 'AppArmor denials: 2 (since')),
            (['--filter.name', '/dev/null'], (0, 'AppArmor denials: 1 (since')),
            (['--filter.name', '/lib/x86_64-linux-gnu/ld-2.27.so'], (0, 'AppArmor denials: 1 (since')),
            (['--filter.name', '/lib/x86_64-linux-gnu/libc-2.27.so'], (0, 'AppArmor denials: 2 (since')),
            (['--filter.name', '/etc/ld.so.cache'], (0, 'AppArmor denials: 1 (since')),
            (['--filter.name', '/usr/lib/locale/locale-archive'], (0, 'AppArmor denials: 1 (since')),
            (['--filter.name', '/usr/bin/file'], (0, 'AppArmor denials: 2 (since')),
            (['--filter.name', '/'], (0, 'AppArmor denials: 10 (since')),
            (['--filter.name', '/.*'], (0, 'AppArmor denials: 10 (since')),
            (['--filter.name', '.*bin.*'], (0, 'AppArmor denials: 4 (since')),
            (['--filter.name', '/(usr/)?bin.*'], (0, 'AppArmor denials: 4 (since')),
            (['--filter.name', '/foo'], (0, 'AppArmor denials: 0 (since')),
        )
        login_params = ['-f', self.test_logfile_last_login, '-l']

        for test in name_tests:
            params = test[0]
            expected = test[1]

            with self.subTest(params=params, expected=expected):
                expected_return_code = expected[0]
                expected_output_has = expected[1]

                return_code, output = cmd(aanotify_bin + login_params + params)
                if 'ERROR: Could not find last login' in output:
                    self.skipTest('Could not find last login')
                result = 'Got return code {}, expected {}\n'.format(return_code, expected_return_code)
                self.assertEqual(expected_return_code, return_code, result + output)
                result = 'Got output "{}", expected "{}"\n'.format(output, expected_output_has)
                self.assertIn(expected_output_has, output, result + output)


class AANotifyDeniedFilterTest(AANotifyBase):

    def test_denied_regex_since_100_days(self):
        denied_tests = (
            (['--filter.denied', 'x'], (0, 'AppArmor denials: 4 (since')),
            (['--filter.denied', 'w'], (0, 'AppArmor denials: 2 (since')),
            (['--filter.denied', 'rm'], (0, 'AppArmor denials: 8 (since')),
            (['--filter.denied', 'r'], (0, 'AppArmor denials: 14 (since')),
            (['--filter.denied', '^r$'], (0, 'AppArmor denials: 6 (since')),
            (['--filter.denied', 'x|w'], (0, 'AppArmor denials: 6 (since')),
            (['--filter.denied', '^(?!rm).*'], (0, 'AppArmor denials: 12 (since')),
            (['--filter.denied', '.(?!m).*'], (0, 'AppArmor denials: 12 (since')),
            (['--filter.denied', 'r.?'], (0, 'AppArmor denials: 14 (since')),
        )
        days_params = ['-f', self.test_logfile_current, '-s', '100']

        for test in denied_tests:
            params = test[0]
            expected = test[1]

            with self.subTest(params=params, expected=expected):
                expected_return_code = expected[0]
                expected_output_has = expected[1]

                return_code, output = cmd(aanotify_bin + days_params + params)
                result = 'Got return code {}, expected {}\n'.format(return_code, expected_return_code)
                self.assertEqual(expected_return_code, return_code, result + output)
                result = 'Got output "{}", expected "{}"\n'.format(output, expected_output_has)
                self.assertIn(expected_output_has, output, result + output)

    @unittest.skipUnless(os.path.isfile('/var/log/wtmp'), 'Requires wtmp on system')
    def test_denied_regex_since_login(self):
        denied_tests = (
            (['--filter.denied', 'x'], (0, 'AppArmor denials: 2 (since')),
            (['--filter.denied', 'w'], (0, 'AppArmor denials: 1 (since')),
            (['--filter.denied', 'rm'], (0, 'AppArmor denials: 4 (since')),
            (['--filter.denied', 'r'], (0, 'AppArmor denials: 7 (since')),
            (['--filter.denied', '^r$'], (0, 'AppArmor denials: 3 (since')),
            (['--filter.denied', 'x|w'], (0, 'AppArmor denials: 3 (since')),
            (['--filter.denied', '^(?!rm).*'], (0, 'AppArmor denials: 6 (since')),
            (['--filter.denied', '.(?!m).*'], (0, 'AppArmor denials: 6 (since')),
            (['--filter.denied', 'r.?'], (0, 'AppArmor denials: 7 (since')),
        )
        login_params = ['-f', self.test_logfile_last_login, '-l']

        for test in denied_tests:
            params = test[0]
            expected = test[1]

            with self.subTest(params=params, expected=expected):
                expected_return_code = expected[0]
                expected_output_has = expected[1]

                return_code, output = cmd(aanotify_bin + login_params + params)
                if 'ERROR: Could not find last login' in output:
                    self.skipTest('Could not find last login')
                result = 'Got return code {}, expected {}\n'.format(return_code, expected_return_code)
                self.assertEqual(expected_return_code, return_code, result + output)
                result = 'Got output "{}", expected "{}"\n'.format(output, expected_output_has)
                self.assertIn(expected_output_has, output, result + output)


setup_aa(aa)  # Wrapper for aa.init_aa()
setup_all_loops(__name__)
if __name__ == '__main__':
    if 'APPARMOR_NOTIFY' in os.environ:
        aanotify_bin = [os.environ['APPARMOR_NOTIFY']]

    if sys.executable:
        aanotify_bin = [sys.executable] + aanotify_bin

    if '__AA_CONFDIR' in os.environ:
        aanotify_bin = aanotify_bin + ['--configdir', os.getenv('__AA_CONFDIR')]

    unittest.main(verbosity=1)



#! /usr/bin/python3
# ------------------------------------------------------------------
#
#    Copyright (C) 2016 Christian Boltz <apparmor@cboltz.de>
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License published by the Free Software Foundation.
#
# ------------------------------------------------------------------

import gettext
import os
import subprocess
import unittest

from apparmor.ui import CMDS, get_translated_hotkey
from common_test import AATest, setup_all_loops


class TestHotkeyConflicts(AATest):
    # check if there are any hotkey conflicts in one of the apparmor-utils translations
    tests = (
        (('CMD_ALLOW', 'CMD_DENY', 'CMD_IGNORE_ENTRY', 'CMD_GLOB', 'CMD_GLOBEXT', 'CMD_NEW', 'CMD_AUDIT_OFF', 'CMD_ABORT', 'CMD_FINISHED'),                 True),  # aa.py available_buttons() with CMD_AUDIT_OFF
        (('CMD_ALLOW', 'CMD_DENY', 'CMD_IGNORE_ENTRY', 'CMD_GLOB', 'CMD_GLOBEXT', 'CMD_NEW', 'CMD_AUDIT_NEW', 'CMD_ABORT', 'CMD_FINISHED'),                 True),  # aa.py available_buttons() with CMD_AUDIT_NEW
        (('CMD_ALLOW', 'CMD_DENY', 'CMD_IGNORE_ENTRY', 'CMD_GLOB', 'CMD_GLOBEXT', 'CMD_NEW', 'CMD_AUDIT_OFF', 'CMD_USER_ON',  'CMD_ABORT', 'CMD_FINISHED'), True),  # aa.py available_buttons() with CMD_AUDIT_OFF and CMD_USER_ON
        (('CMD_ALLOW', 'CMD_DENY', 'CMD_IGNORE_ENTRY', 'CMD_GLOB', 'CMD_GLOBEXT', 'CMD_NEW', 'CMD_AUDIT_OFF', 'CMD_USER_OFF', 'CMD_ABORT', 'CMD_FINISHED'), True),  # aa.py available_buttons() with CMD_AUDIT_OFF and CMD_USER_OFF
        (('CMD_ALLOW', 'CMD_DENY', 'CMD_IGNORE_ENTRY', 'CMD_GLOB', 'CMD_GLOBEXT', 'CMD_NEW', 'CMD_AUDIT_NEW', 'CMD_USER_ON',  'CMD_ABORT', 'CMD_FINISHED'), True),  # aa.py available_buttons() with CMD_AUDIT_NEW and CMD_USER_ON
        (('CMD_ALLOW', 'CMD_DENY', 'CMD_IGNORE_ENTRY', 'CMD_GLOB', 'CMD_GLOBEXT', 'CMD_NEW', 'CMD_AUDIT_NEW', 'CMD_USER_OFF', 'CMD_ABORT', 'CMD_FINISHED'), True),  # aa.py available_buttons() with CMD_AUDIT_NEW and CMD_USER_OFF
        (('CMD_SAVE_CHANGES', 'CMD_SAVE_SELECTED', 'CMD_VIEW_CHANGES', 'CMD_VIEW_CHANGES_CLEAN', 'CMD_ABORT'),                                              True),  # aa.py save_profiles()
        (('CMD_VIEW_PROFILE', 'CMD_USE_PROFILE', 'CMD_CREATE_PROFILE', 'CMD_ABORT'),                                                                        True),  # aa.py get_profile()
        (('CMD_ix', 'CMD_pix', 'CMD_cix', 'CMD_nix', 'CMD_EXEC_IX_OFF', 'CMD_ux', 'CMD_DENY', 'CMD_ABORT', 'CMD_FINISHED', 'CMD_IGNORE_ENTRY'),             True),  # aa.py build_x_functions() with exec_toggle
        (('CMD_ix', 'CMD_cx', 'CMD_px', 'CMD_nx', 'CMD_ux', 'CMD_EXEC_IX_ON', 'CMD_DENY', 'CMD_ABORT', 'CMD_FINISHED', 'CMD_IGNORE_ENTRY'),                 True),  # aa.py build_x_functions() without exec_toggle
        (('CMD_ADDHAT', 'CMD_USEDEFAULT', 'CMD_DENY', 'CMD_ABORT', 'CMD_FINISHED'),                                                                         True),  # aa.py ask_addhat()
        (('CMD_YES', 'CMD_NO', 'CMD_CANCEL'),                                                                                                               True),  # ui.py UI_YesNo() and UI_YesNoCancel
        (('CMD_SAVE_CHANGES', 'CMD_VIEW_CHANGES', 'CMD_ABORT', 'CMD_IGNORE_ENTRY'),                                                                         True),  # aa-mergeprof act()
        (('CMD_ALLOW', 'CMD_ABORT'),                                                                                                                        True),  # aa-mergeprof conflict_mode()
        (('CMD_ADDSUBPROFILE', 'CMD_DENY', 'CMD_ABORT', 'CMD_FINISHED'),                                                                                    True),  # aa-mergeprof ask_the_questions() - new subprofile
        (('CMD_ADDHAT', 'CMD_DENY', 'CMD_ABORT', 'CMD_FINISHED'),                                                                                           True),  # aa-mergeprof ask_the_questions() - new hat
    )

    def _run_test(self, params, expected):
        self.createTmpdir()

        subprocess.run("make -C ../po >/dev/null", shell=True, check=True)
        subprocess.run(
            "DESTDIR={} NAME=apparmor-utils make -C ../po install >/dev/null".format(self.tmpdir),
            shell=True, check=True,
        )

        self.localedir = '{}/usr/share/locale'.format(self.tmpdir)

        self.languages = os.listdir(self.localedir)

        # make sure we found all translations
        if len(self.languages) < 15:
            raise Exception('None or not all languages found, only {}'.format(self.languages))

        # we also want to detect hotkey conflicts in the untranslated english strings
        self.languages.append('C')

        conflicts = []

        for language in self.languages:
            with self.subTest(language=language):
                t = gettext.translation(
                    'apparmor-utils', fallback=True,
                    localedir=self.localedir, languages=(language,),
                )

                keys = {}
                all_keys = []
                found_conflict = ''

                for key in params:
                    text = t.gettext(CMDS[key])
                    all_keys.append(text)
                    hotkey = get_translated_hotkey(text)

                    if keys.get(hotkey):
                        found_conflict = "Hotkey conflict for {}: '{}' and '{}'".format(language, keys[hotkey], text)
                    else:
                        keys[hotkey] = text

                if found_conflict:
                    conflicts.append("{}\n    {}'".format(found_conflict, '  '.join(all_keys)))

        if conflicts:
            raise Exception('\n'.join(conflicts))


setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)



#!/usr/bin/python3
# ----------------------------------------------------------------------
#    Copyright (C) 2024 Canonical, Ltd.
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
# ----------------------------------------------------------------------

import unittest
from common_test import AATest, setup_all_loops

from apparmor.common import AppArmorException
from apparmor.translations import init_translation

from apparmor.rule.unix import UnixRule

_ = init_translation()


class UnixTestParse(AATest):

    tests = (
        #                   Rule                                     Accesses           Rule conds                        Local expr                    Peer expr                       Audit  Deny   Allow  Comment
        ('unix,',                                           UnixRule(UnixRule.ALL,      UnixRule.ALL,                     UnixRule.ALL,                 UnixRule.ALL,                   False, False, False, '')),
        ('unix rw,',                                        UnixRule('rw',              UnixRule.ALL,                     UnixRule.ALL,                 UnixRule.ALL,                   False, False, False, '')),
        ('unix (accept, rw),',                              UnixRule(('accept', 'rw'),  UnixRule.ALL,                     UnixRule.ALL,                 UnixRule.ALL,                   False, False, False, '')),
        ('unix peer=(addr=AA label=bb),',                   UnixRule(UnixRule.ALL,      UnixRule.ALL,                     UnixRule.ALL,                 {'addr': 'AA', 'label': 'bb'},  False, False, False, '')),
        ('unix opt=AA label=bb,',                           UnixRule(UnixRule.ALL,      UnixRule.ALL,                     {'opt': 'AA', 'label': 'bb'}, UnixRule.ALL,                   False, False, False, '')),
        ('unix (accept rw) type=AA protocol=BB,',           UnixRule(('accept', 'rw'),  {'type': 'AA', 'protocol': 'BB'}, UnixRule.ALL,                 UnixRule.ALL,                   False, False, False, '')),
        ('unix (accept, rw) protocol=AA type=BB,',          UnixRule(('accept', 'rw'),  {'type': 'BB', 'protocol': 'AA'}, UnixRule.ALL,                 UnixRule.ALL,                   False, False, False, '')),
        ('unix shutdown addr=@srv,',                        UnixRule('shutdown',        UnixRule.ALL,                     {'addr': '@srv'},             UnixRule.ALL,                   False, False, False, '')),
        ('unix send addr=@foo{a,b} peer=(label=splat),',    UnixRule('send',            UnixRule.ALL,                     {'addr': '@foo{a,b}'},        {'label': 'splat'},             False, False, False, '')),
        ('unix peer=(addr=@/tmp/foo-??????),',              UnixRule(UnixRule.ALL,      UnixRule.ALL,                     UnixRule.ALL,                 {'addr': '@/tmp/foo-??????'},   False, False, False, '')),
        ('unix peer=(addr="@/tmp/f o-??????"),',            UnixRule(UnixRule.ALL,      UnixRule.ALL,                     UnixRule.ALL,                 {'addr': '@/tmp/f o-??????'},   False, False, False, '')),
        ('unix peer=(addr=@/tmp/foo-*),',                   UnixRule(UnixRule.ALL,      UnixRule.ALL,                     UnixRule.ALL,                 {'addr': '@/tmp/foo-*'},        False, False, False, '')),
        ('unix (accept, rw) protocol=AA type=BB opt=AA label=bb peer=(addr=a label=bb),',
                                                            UnixRule(('accept', 'rw'),  {'type': 'BB', 'protocol': 'AA'}, {'opt': 'AA', 'label': 'bb'}, {'addr': 'a', 'label': 'bb'},   False, False, False, '')),  # noqa: E127
        ('unix peer=( label=la, addr="@/h"),',              UnixRule(UnixRule.ALL,      UnixRule.ALL,                     UnixRule.ALL,                {'addr': '@/h', 'label': 'la,'}, False, False, False, '')),
        ('unix peer=(addr="@/h o", label="l a"),',          UnixRule(UnixRule.ALL,      UnixRule.ALL,                     UnixRule.ALL,              {'addr': '@/h o', 'label': 'l a'}, False, False, False, '')),
        ('unix addr="@/h" label=la,',                       UnixRule(UnixRule.ALL,      UnixRule.ALL,                     {'addr': '@/h', 'label': 'la'},    UnixRule.ALL,              False, False, False, '')),
        ('unix addr="@/h o" label="l a",',                  UnixRule(UnixRule.ALL,      UnixRule.ALL,                     {'addr': '@/h o', 'label': 'l a'}, UnixRule.ALL,              False, False, False, '')),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(UnixRule.match(rawrule))
        obj = UnixRule.create_instance(rawrule)
        expected.raw_rule = rawrule.strip()
        self.assertTrue(obj.is_equal(expected, True), f'\n  {rawrule}   expected,\n  {obj.get_clean()}   returned by obj.get_clean()\n  {expected.get_clean()}   returned by expected.get_clean()')

    def test_diff_local(self):
        obj1 = UnixRule('send', UnixRule.ALL, {'addr': 'foo'}, UnixRule.ALL, )
        obj2 = UnixRule('send', UnixRule.ALL, UnixRule.ALL, {'addr': 'bar'})
        self.assertFalse(obj1.is_equal(obj2, False))

    def test_diff_peer(self):
        obj1 = UnixRule('send', UnixRule.ALL, UnixRule.ALL, {'addr': 'foo'})
        obj2 = UnixRule('send', UnixRule.ALL, UnixRule.ALL, {'addr': 'bar'})
        self.assertFalse(obj1.is_equal(obj2, False))


class UnixTestParseInvalid(AATest):
    tests = (
        ('unix invalid,',   AppArmorException),
        ('unix (invalid),', AppArmorException),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(UnixRule.match(rawrule))  # the above invalid rules still match the main regex!
        with self.assertRaises(expected):
            UnixRule.create_instance(rawrule)

    def test_parse_fail(self):
        with self.assertRaises(AppArmorException):
            UnixRule.create_instance('foo,')

    def test_invalid_key(self):
        with self.assertRaises(AppArmorException):
            UnixRule('send', UnixRule.ALL, {'invalid': 'whatever'}, UnixRule.ALL,  False, False, False, '')

    def test_invalid_access(self):
        with self.assertRaises(AppArmorException):
            UnixRule('invalid', UnixRule.ALL, UnixRule.ALL, UnixRule.ALL,  False, False, False, '')

    def test_invalid_access2(self):
        with self.assertRaises(AppArmorException):
            UnixRule(('rw', 'invalid'), UnixRule.ALL, UnixRule.ALL, UnixRule.ALL,  False, False, False, '')

    def test_invalid_peer_expr(self):
        with self.assertRaises(AppArmorException):
            UnixRule('create', UnixRule.ALL, UnixRule.ALL, {'addr': 'foo'}, False, False, False, '')


class UnixIsCoveredTest(AATest):
    def test_is_covered(self):
        obj = UnixRule(('accept', 'rw'), {'type': 'F*', 'protocol': 'AA'}, {'addr': 'AA'}, {'addr': 'AA', 'label': 'bb'})
        tests = [
            (('accept',),       {'type': 'F*', 'protocol': 'AA'},   {'opt': 'AA', 'label': 'bb'},   {'addr': 'AA', 'label': 'bb'}),
            (('accept', 'rw'),  {'type': 'F*'},                     {'opt': 'AA', 'label': 'bb'},   {'addr': 'AA', 'label': 'bb'}),
            (('accept', 'rw'),  {'type': 'Foo'},                    {'addr': 'AA'},                 {'addr': 'AA', 'label': 'bb'}),
            (('accept', 'rw'),  {'type': 'Foo'},                    {'addr': 'AA', 'opt': 'BB'},    {'addr': 'AA', 'label': 'bb'})
        ]
        for test in tests:
            self.assertTrue(obj.is_covered(UnixRule(*test)))
            self.assertFalse(obj.is_equal(UnixRule(*test)))

    def test_is_covered2(self):
        obj = UnixRule(('accept', 'rw'), UnixRule.ALL, {'addr': 'AA'}, {'addr': 'AA', 'label': 'bb'})
        tests = [
            (('accept',),       {'type': 'F*', 'protocol': 'AA'},   {'opt': 'AA', 'label': 'bb'},   {'addr': 'AA', 'label': 'bb'}),
            (('accept', 'rw'),  {'type': 'F*'},                     {'opt': 'AA', 'label': 'bb'},   {'addr': 'AA', 'label': 'bb'}),
            (('accept', 'rw'),  {'type': 'Foo'},                    {'addr': 'AA'},                 {'addr': 'AA', 'label': 'bb'}),
            (('accept', 'rw'),  {'type': 'Foo'},                    {'addr': 'AA', 'opt': 'BB'},    {'addr': 'AA', 'label': 'bb'})
        ]
        for test in tests:
            self.assertTrue(obj.is_covered(UnixRule(*test)))
            self.assertFalse(obj.is_equal(UnixRule(*test)))

    def test_is_not_covered(self):
        obj = UnixRule(('accept', 'rw'), {'type': 'F'}, {'opt': 'AA'}, {'addr': 'AA', 'label': 'bb'})
        tests = [
            (('r',),            {'type': 'F*', 'protocol': 'AA'},   {'opt': 'AA', 'label': 'bb'},   {'addr': 'AA', 'label': 'bb'}),
            (('accept', 'rw'),  {'type': 'B'},                      {'opt': 'AA', 'label': 'bb'},   {'addr': 'AA', 'label': 'bb'}),
            (('accept', 'rw'),  {'type': 'F'},                      {'opt': 'AA', 'label': 'bb'},   UnixRule.ALL),
            (('accept', 'rw'),  {'type': 'F'},                      {'opt': 'notcovered'},          {'addr': 'AA', 'label': 'bb'}),
            (('accept', 'rw'),  {'type': 'F'},                      {'opt': 'AA'},                  {'addr': 'notcovered'}),
        ]
        for test in tests:
            self.assertFalse(obj.is_covered(UnixRule(*test)), test)
            self.assertFalse(obj.is_equal(UnixRule(*test)))

    def test_is_covered_priority(self):
        obj = UnixRule(('accept', 'rw'), {'type': 'F*', 'protocol': 'AA'}, {'addr': 'AA'}, {'addr': 'AA', 'label': 'bb'}, priority=0)
        prio_obj = UnixRule(('accept', 'rw'), {'type': 'F*', 'protocol': 'AA'}, {'addr': 'AA'}, {'addr': 'AA', 'label': 'bb'}, priority=1)

        self.assertFalse(obj.is_covered(prio_obj))
        self.assertTrue(prio_obj.is_covered(obj))


class UnixLogprofHeaderTest(AATest):
    tests = (
        ('unix,',                                           [_('Accesses'), 'ALL',  _('Rule'), 'ALL', _('Local'), 'ALL',                        _('Peer'), 'ALL']),
        ('unix rw,',                                        [_('Accesses'), 'rw',   _('Rule'), 'ALL', _('Local'), 'ALL',                        _('Peer'), 'ALL']),
        ('unix send addr=@foo{one,two peer=(label=splat),', [_('Accesses'), 'send', _('Rule'), 'ALL', _('Local'), {'addr': '@foo{one,two'},     _('Peer'), {'label': 'splat'}])
    )

    def _run_test(self, params, expected):
        obj = UnixRule.create_instance(params)
        self.assertEqual(obj.logprof_header(), expected)


class UnixTestGlob(AATest):
    def test_glob(self):
        glob_list = [(
            'unix (accept, rw) type=BB protocol=AA label=bb opt=AA peer=(addr=a label=bb),',
            'unix (accept, rw) type=BB protocol=AA label=bb opt=AA,',
            'unix (accept, rw) type=BB protocol=AA,',
            'unix (accept, rw),',
            'unix,',
        )]
        for globs in glob_list:
            for i in range(len(globs) - 1):
                rule = UnixRule.create_instance(globs[i])
                rule.glob()
                self.assertEqual(rule.get_clean(), globs[i + 1])


class UnixTestClean(AATest):
    tests = (
        ('     audit  unix                                                                                                                   ,    # foo  ', 'audit unix, # foo'),
        ('     audit deny unix                                                   label  =  foo                                               ,           ', 'audit deny unix label=foo,'),
        ('     audit allow unix                                                  peer  =  (addr  =  a)                                       ,    # foo  ', 'audit allow unix peer=(addr=a), # foo'),
        ('     deny unix                                                   type  =  foo                                                      ,           ', 'deny unix type=foo,'),
        ('     allow unix                                                              peer  =  (label=bb)                                   ,    # foo  ', 'allow unix peer=(label=bb), # foo'),
        ('     unix                                                                                                                          ,    # foo  ', 'unix, # foo'),
        ('     unix                                                   addr  =  foo                                                           ,           ', 'unix addr=foo,'),
        ('     unix    (  accept  , rw)  protocol  =  AA  type =  BB  opt  =  myopt  label  =  bb peer  =  (addr  =  a label  =  bb )        ,           ', 'unix (accept, rw) type=BB protocol=AA label=bb opt=myopt peer=(addr=a label=bb),'),
        ('priority=-42 unix    (  accept  , rw)  protocol  =  AA  type =  BB  opt  =  myopt  label  =  bb peer  =  (addr  =  a label  =  bb ),           ', 'priority=-42 unix (accept, rw) type=BB protocol=AA label=bb opt=myopt peer=(addr=a label=bb),'),
        ('priority = 0 unix    (  accept  , rw)  protocol  =  AA  type =  BB  opt  =  myopt  label  =  bb peer  =  (addr  =  a label  =  bb ),           ', 'priority=0 unix (accept, rw) type=BB protocol=AA label=bb opt=myopt peer=(addr=a label=bb),'),
        ('priority=211 unix    (  accept  , rw)  protocol  =  AA  type =  BB  opt  =  myopt  label  =  bb peer  =  (addr  =  a label  =  bb ),           ', 'priority=211 unix (accept, rw) type=BB protocol=AA label=bb opt=myopt peer=(addr=a label=bb),'),
        ('priority=+45 unix    (  accept  , rw)  protocol  =  AA  type =  BB  opt  =  myopt  label  =  bb peer  =  (addr  =  a label  =  bb ),           ', 'priority=45 unix (accept, rw) type=BB protocol=AA label=bb opt=myopt peer=(addr=a label=bb),'),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(UnixRule.match(rawrule))
        obj = UnixRule.create_instance(rawrule)
        clean = obj.get_clean()
        raw = obj.get_raw()

        self.assertEqual(expected, clean, 'unexpected clean rule')
        self.assertEqual(rawrule.strip(), raw, 'unexpected raw rule')


setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)
